# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==3.5.0",
#     "huggingface-hub==0.24.0",
#     "pandas==2.2.0",
#     "requests==2.32.3",
# ]
# ///

"""Create the Czech News Summarization dataset."""

import pandas as pd
from datasets import Dataset, DatasetDict, Split, load_dataset
from huggingface_hub import HfApi

from .constants import MAX_NUM_CHARS_IN_ARTICLE, MIN_NUM_CHARS_IN_ARTICLE


def main() -> None:
    """Create the Czech News Summarization dataset and upload to HF Hub."""
    dataset_id = "hynky/czech_news_dataset_v2"

    # Load the dataset with all splits
    dataset = load_dataset(dataset_id, split=None, token=True)
    assert isinstance(dataset, DatasetDict)

    # Rename columns
    dataset = dataset.rename_columns(
        column_mapping=dict(content="text", brief="target_text")
    )
    dataset = dataset.select_columns(["text", "target_text"])

    # Convert each split to pandas DataFrame
    train_df = dataset["train"].to_pandas()
    val_df = dataset["validation"].to_pandas()
    test_df = dataset["test"].to_pandas()

    assert isinstance(train_df, pd.DataFrame)
    assert isinstance(val_df, pd.DataFrame)
    assert isinstance(test_df, pd.DataFrame)

    # Sample from each split, to avoid processing the full dataset
    train_df = train_df.sample(n=2000, random_state=4242).reset_index(drop=True)
    val_df = val_df.sample(n=500, random_state=4242).reset_index(drop=True)
    test_df = test_df.sample(n=3000, random_state=4242).reset_index(drop=True)

    # Only work with samples where the text is not very large or small
    train_lengths = train_df.text.str.len()
    val_lengths = val_df.text.str.len()
    test_lengths = test_df.text.str.len()
    lower_bound = MIN_NUM_CHARS_IN_ARTICLE
    upper_bound = MAX_NUM_CHARS_IN_ARTICLE
    train_df = train_df.loc[train_lengths.between(lower_bound, upper_bound)]
    val_df = val_df.loc[val_lengths.between(lower_bound, upper_bound)]
    test_df = test_df.loc[test_lengths.between(lower_bound, upper_bound)]

    # Make final splits
    train_size = 1024
    val_size = 256
    test_size = 2048

    train_df = train_df.sample(n=train_size, random_state=4242).reset_index(drop=True)
    val_df = val_df.sample(n=val_size, random_state=4242).reset_index(drop=True)
    test_df = test_df.sample(n=test_size, random_state=4242).reset_index(drop=True)

    # Collect datasets in a dataset dictionary
    dataset = DatasetDict(
        {
            "train": Dataset.from_pandas(train_df, split=Split.TRAIN),
            "val": Dataset.from_pandas(val_df, split=Split.VALIDATION),
            "test": Dataset.from_pandas(test_df, split=Split.TEST),
        }
    )

    dataset_id = "EuroEval/czech-news-mini"
    # Push the dataset to the Hugging Face Hub
    HfApi().delete_repo(dataset_id, repo_type="dataset", missing_ok=True)
    dataset.push_to_hub(dataset_id, private=True)


if __name__ == "__main__":
    main()



"""Templates for the classification task."""

import typing as t

from ..data_models import PromptConfig
from ..languages import (
    BULGARIAN,
    CZECH,
    DANISH,
    DUTCH,
    ENGLISH,
    ESTONIAN,
    FAROESE,
    FINNISH,
    FRENCH,
    GERMAN,
    GREEK,
    ICELANDIC,
    ITALIAN,
    LATVIAN,
    LITHUANIAN,
    NORWEGIAN,
    NORWEGIAN_BOKMÅL,
    NORWEGIAN_NYNORSK,
    POLISH,
    PORTUGUESE,
    SLOVAK,
    SPANISH,
    SWEDISH,
    UKRAINIAN,
)

if t.TYPE_CHECKING:
    from ..languages import Language

CLASSIFICATION_TEMPLATES: dict["Language", PromptConfig] = {
    ENGLISH: PromptConfig(
        default_prompt_prefix="The following are texts and their labels.",
        default_prompt_template="Text: {text}\nLabel: {label}",
        default_instruction_prompt="Here is a text:\n'{text}'.\n\nClassify the text "
        "into the categories {labels_str}, and reply with only the label.",
        default_prompt_label_mapping="auto",
    ),
    BULGARIAN: PromptConfig(
        default_prompt_prefix="Следват текстове и техните етикети.",
        default_prompt_template="Текст: {text}\nЕтикет: {label}",
        default_instruction_prompt="Ето един текст:\n'{text}'.\n\nКласифицирайте "
        "текста в категориите {labels_str} и отговорете само с етикета.",
        default_prompt_label_mapping="auto",
    ),
    CZECH: PromptConfig(
        default_prompt_prefix="Následují texty a jejich štítky.",
        default_prompt_template="Text: {text}\nŠtítek: {label}",
        default_instruction_prompt="Zde je text:\n'{text}'.\n\nKlasifikujte text do "
        "kategorií {labels_str} a odpovězte pouze štítkem.",
        default_prompt_label_mapping="auto",
    ),
    DANISH: PromptConfig(
        default_prompt_prefix="Følgende er tekster og deres etiketter.",
        default_prompt_template="Tekst: {text}\nEtiket: {label}",
        default_instruction_prompt="Her er en tekst:\n'{text}'.\n\nKlassificer teksten "
        "i kategorierne {labels_str}, og svar kun med etiketten.",
        default_prompt_label_mapping="auto",
    ),
    GERMAN: PromptConfig(
        default_prompt_prefix="Im Folgenden sind Texte und ihre Labels aufgeführt.",
        default_prompt_template="Text: {text}\nLabel: {label}",
        default_instruction_prompt="Hier ist ein Text:\n'{text}'.\n\nKlassifiziere den "
        "Text in die Kategorien {labels_str} und antworte nur mit dem Label.",
        default_prompt_label_mapping="auto",
    ),
    GREEK: PromptConfig(
        default_prompt_prefix="Ακολουθούν κείμενα και οι ετικέτες τους.",
        default_prompt_template="Κείμενο: {text}\nΕτικέτα: {label}",
        default_instruction_prompt="Εδώ είναι ένα κείμενο:\n'{text}'.\n\n"
        "Κατηγοριοποιήστε το κείμενο στις κατηγορίες {labels_str} και απαντήστε μόνο "
        "με την ετικέτα.",
        default_prompt_label_mapping="auto",
    ),
    SPANISH: PromptConfig(
        default_prompt_prefix="A continuación se presentan textos y sus etiquetas.",
        default_prompt_template="Texto: {text}\nEtiqueta: {label}",
        default_instruction_prompt="Aquí hay un texto:\n'{text}'.\n\nClasifica el "
        "texto en las categorías {labels_str} y responde solo con la etiqueta.",
        default_prompt_label_mapping="auto",
    ),
    ESTONIAN: PromptConfig(
        default_prompt_prefix="Järgnevad on tekstid ja nende sildid.",
        default_prompt_template="Tekst: {text}\nSilt: {label}",
        default_instruction_prompt="Siin on tekst:\n'{text}'.\n\nKlassifitseeri tekst "
        "kategooriatesse {labels_str} ja vasta ainult sildiga.",
        default_prompt_label_mapping="auto",
    ),
    FINNISH: PromptConfig(
        default_prompt_prefix="Seuraavassa on tekstejä ja niiden tunnisteita.",
        default_prompt_template="Teksti: {text}\nTunniste: {label}",
        default_instruction_prompt="Tässä on teksti:\n'{text}'.\n\nLuokittele teksti "
        "kategorioihin {labels_str} ja vastaa vain tunnisteella.",
        default_prompt_label_mapping="auto",
    ),
    FAROESE: PromptConfig(
        default_prompt_prefix="Hér eru tekster og teirra etikettir.",
        default_prompt_template="Tekstur: {text}\nEtikettur: {label}",
        default_instruction_prompt="Her er ein tekstur:\n'{text}'.\n\nFlokka teksturin "
        "í bólkar {labels_str} og svara bert við etikettinum.",
        default_prompt_label_mapping="auto",
    ),
    FRENCH: PromptConfig(
        default_prompt_prefix="Voici des textes et leurs étiquettes.",
        default_prompt_template="Texte : {text}\nÉtiquette : {label}",
        default_instruction_prompt="Voici un texte :\n'{text}'.\n\nClassifiez le texte "
        "dans les catégories {labels_str} et répondez uniquement avec l'étiquette.",
        default_prompt_label_mapping="auto",
    ),
    ICELANDIC: PromptConfig(
        default_prompt_prefix="Hér fyrir neðan eru textar og merkingar þeirra.",
        default_prompt_template="Texti: {text}\nMerking: {label}",
        default_instruction_prompt="Hér er texti:\n'{text}'.\n\nFlokkaðu textann "
        "í flokkana {labels_str} og svaraðu aðeins með merkingenni.",
        default_prompt_label_mapping="auto",
    ),
    ITALIAN: PromptConfig(
        default_prompt_prefix="Di seguito sono riportati testi e le loro etichette.",
        default_prompt_template="Testo: {text}\nEtichetta: {label}",
        default_instruction_prompt="Ecco un testo:\n'{text}'.\n\nClassifica il testo "
        "nelle categorie {labels_str} e rispondi solo con l'etichetta.",
        default_prompt_label_mapping="auto",
    ),
    LITHUANIAN: PromptConfig(
        default_prompt_prefix="Toliau pateikiami tekstai ir jų etiketės.",
        default_prompt_template="Tekstas: {text}\nEtiketė: {label}",
        default_instruction_prompt="Štai tekstas:\n'{text}'.\n\nKlasifikuokite tekstą "
        "į kategorijas {labels_str} ir atsakykite tik etiketę.",
        default_prompt_label_mapping="auto",
    ),
    LATVIAN: PromptConfig(
        default_prompt_prefix="Turpmāk ir teksti un to etiķetes.",
        default_prompt_template="Teksts: {text}\nEtiķete: {label}",
        default_instruction_prompt="Šeit ir teksts:\n'{text}'.\n\nKlasificējiet tekstu "
        "kategorijās {labels_str} un atbildiet tikai ar etiķeti.",
        default_prompt_label_mapping="auto",
    ),
    NORWEGIAN_BOKMÅL: PromptConfig(
        default_prompt_prefix="Følgende er tekster og deres etiketter.",
        default_prompt_template="Tekst: {text}\nEtikett: {label}",
        default_instruction_prompt="Her er en tekst:\n'{text}'.\n\nKlassifiser teksten "
        "i kategoriene {labels_str}, og svar kun med etiketten.",
        default_prompt_label_mapping="auto",
    ),
    DUTCH: PromptConfig(
        default_prompt_prefix="Hieronder volgen teksten en hun labels.",
        default_prompt_template="Tekst: {text}\nLabel: {label}",
        default_instruction_prompt="Hier is een tekst:\n'{text}'.\n\nClassificeer de "
        "tekst in de categorieën {labels_str} en antwoord alleen met het label.",
        default_prompt_label_mapping="auto",
    ),
    NORWEGIAN_NYNORSK: PromptConfig(
        default_prompt_prefix="Følgjande er tekstar og deira etikettar.",
        default_prompt_template="Tekst: {text}\nEtikett: {label}",
        default_instruction_prompt="Her er ein tekst:\n'{text}'.\n\nKlassifiser "
        "teksten i kategoriane {labels_str}, og svar berre med etiketten.",
        default_prompt_label_mapping="auto",
    ),
    NORWEGIAN: PromptConfig(
        default_prompt_prefix="Følgende er tekster og deres etiketter.",
        default_prompt_template="Tekst: {text}\nEtikett: {label}",
        default_instruction_prompt="Her er en tekst:\n'{text}'.\n\nKlassifiser teksten "
        "i kategoriene {labels_str}, og svar kun med etiketten.",
        default_prompt_label_mapping="auto",
    ),
    POLISH: PromptConfig(
        default_prompt_prefix="Poniżej znajdują się teksty i ich etykiety.",
        default_prompt_template="Tekst: {text}\nEtykieta: {label}",
        default_instruction_prompt="Oto tekst:\n'{text}'.\n\nSklasyfikuj tekst do "
        "kategorii {labels_str} i odpowiedz tylko etykietą.",
        default_prompt_label_mapping="auto",
    ),
    PORTUGUESE: PromptConfig(
        default_prompt_prefix="A seguir estão textos e seus rótulos.",
        default_prompt_template="Texto: {text}\nRótulo: {label}",
        default_instruction_prompt="Aqui está um texto:\n'{text}'.\n\nClassifique o "
        "texto nas categorias {labels_str} e responda apenas com o rótulo.",
        default_prompt_label_mapping="auto",
    ),
    SLOVAK: PromptConfig(
        default_prompt_prefix="Nasledujú texty a ich štítky.",
        default_prompt_template="Text: {text}\nŠtítok: {label}",
        default_instruction_prompt="Tu je text:\n'{text}'.\n\nKlasifikujte text do "
        "kategorií {labels_str} a odpovedzte iba štítkom.",
        default_prompt_label_mapping="auto",
    ),
    SWEDISH: PromptConfig(
        default_prompt_prefix="Följande är texter och deras etiketter.",
        default_prompt_template="Text: {text}\nEtikett: {label}",
        default_instruction_prompt="Här är en text:\n'{text}'.\n\nKlassificera texten "
        "i kategorierna {labels_str} och svara endast med etiketten.",
        default_prompt_label_mapping="auto",
    ),
    UKRAINIAN: PromptConfig(
        default_prompt_prefix="Нижче наведено тексти та їхні позначки.",
        default_prompt_template="Текст: {text}\nПозначка: {label}",
        default_instruction_prompt="Ось текст:\n'{text}'.\n\nКласифікуйте текст у "
        "категорії {labels_str} і відповідайте лише позначкою.",
        default_prompt_label_mapping="auto",
    ),
}



"""All Albanian dataset configurations used in EuroEval."""

from ..data_models import DatasetConfig
from ..languages import ALBANIAN
from ..tasks import COMMON_SENSE, KNOW, LA, NER, RC, SENT, SUMM

### Official datasets ###

MMS_SQ_CONFIG = DatasetConfig(
    name="mms-sq",
    pretty_name="MMS-sq",
    source="EuroEval/mms-sq-mini",
    task=SENT,
    languages=[ALBANIAN],
)

SCALA_SQ_CONFIG = DatasetConfig(
    name="scala-sq",
    pretty_name="ScaLA-sq",
    source="EuroEval/scala-sq",
    task=LA,
    languages=[ALBANIAN],
)

WIKIANN_SQ_CONFIG = DatasetConfig(
    name="wikiann-sq",
    pretty_name="WikiANN-sq",
    source="EuroEval/wikiann-sq-mini",
    task=NER,
    languages=[ALBANIAN],
)

MULTI_WIKI_QA_SQ_CONFIG = DatasetConfig(
    name="multi-wiki-qa-sq",
    pretty_name="MultiWikiQA-sq",
    source="EuroEval/multi-wiki-qa-sq-mini",
    task=RC,
    languages=[ALBANIAN],
)

LR_SUM_SQ_CONFIG = DatasetConfig(
    name="lr-sum-sq",
    pretty_name="LRSum-sq",
    source="EuroEval/lr-sum-sq-mini",
    task=SUMM,
    languages=[ALBANIAN],
)

GLOBAL_MMLU_LITE_SQ_CONFIG = DatasetConfig(
    name="global-mmlu-lite-sq",
    pretty_name="GlobalMMLULite-sq",
    source="EuroEval/global-mmlu-lite-sq",
    task=KNOW,
    languages=[ALBANIAN],
)

WINOGRANDE_SQ_CONFIG = DatasetConfig(
    name="winogrande-sq",
    pretty_name="Winogrande-sq",
    source="EuroEval/winogrande-sq",
    task=COMMON_SENSE,
    languages=[ALBANIAN],
    _labels=["a", "b"],
)



"""All Polish dataset configurations used in EuroEval."""

from ..data_models import DatasetConfig
from ..languages import POLISH
from ..tasks import COMMON_SENSE, EUROPEAN_VALUES, KNOW, LA, NER, RC, SENT, SUMM

### Official datasets ###

POLEMO2_CONFIG = DatasetConfig(
    name="polemo2",
    pretty_name="Polemo2",
    source="EuroEval/polemo2-mini",
    task=SENT,
    languages=[POLISH],
)

SCALA_PL_CONFIG = DatasetConfig(
    name="scala-pl",
    pretty_name="ScaLA-pl",
    source="EuroEval/scala-pl",
    task=LA,
    languages=[POLISH],
)

KPWR_NER_CONFIG = DatasetConfig(
    name="kpwr-ner",
    pretty_name="KPWr-NER",
    source="EuroEval/kpwr-ner",
    task=NER,
    languages=[POLISH],
)

POQUAD_CONFIG = DatasetConfig(
    name="poquad",
    pretty_name="PoQuAD",
    source="EuroEval/poquad-mini",
    task=RC,
    languages=[POLISH],
)

PSC_CONFIG = DatasetConfig(
    name="psc",
    pretty_name="PSC",
    source="EuroEval/psc-mini",
    task=SUMM,
    languages=[POLISH],
)

LLMZSZL_CONFIG = DatasetConfig(
    name="llmzszl",
    pretty_name="LLMzSzŁ",
    source="EuroEval/llmzszl-mini",
    task=KNOW,
    languages=[POLISH],
)

WINOGRANDE_PL_CONFIG = DatasetConfig(
    name="winogrande-pl",
    pretty_name="Winogrande-pl",
    source="EuroEval/winogrande-pl",
    task=COMMON_SENSE,
    languages=[POLISH],
    _labels=["a", "b"],
)

VALEU_PL_CONFIG = DatasetConfig(
    name="valeu-pl",
    pretty_name="VaLEU-pl",
    source="EuroEval/european-values-pl",
    task=EUROPEAN_VALUES,
    languages=[POLISH],
    splits=["test"],
    bootstrap_samples=False,
    _instruction_prompt="{text}",
)


### Unofficial datasets ###

MULTI_WIKI_QA_PL_CONFIG = DatasetConfig(
    name="multi-wiki-qa-pl",
    pretty_name="MultiWikiQA-pl",
    source="EuroEval/multi-wiki-qa-pl-mini",
    task=RC,
    languages=[POLISH],
    unofficial=True,
)

GOLDENSWAG_PL_CONFIG = DatasetConfig(
    name="goldenswag-pl",
    pretty_name="GoldenSwag-pl",
    source="EuroEval/goldenswag-pl-mini",
    task=COMMON_SENSE,
    languages=[POLISH],
    unofficial=True,
)



# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==3.5.0",
#     "huggingface-hub==0.24.0",
#     "pandas==2.2.0",
#     "requests==2.32.3",
#     "scikit-learn<1.6.0",
# ]
# ///

"""Create the SweReC-mini sentiment dataset and upload it to the HF Hub."""

import io

import pandas as pd
import requests
from datasets import Split
from datasets.arrow_dataset import Dataset
from datasets.dataset_dict import DatasetDict
from huggingface_hub.hf_api import HfApi
from sklearn.model_selection import train_test_split

from .constants import MAX_NUM_CHARS_IN_DOCUMENT, MIN_NUM_CHARS_IN_DOCUMENT  # noqa


def main() -> None:
    """Create the SweReC-mini sentiment dataset and upload it to the HF Hub."""
    # Define the base download URL
    url = (
        "https://github.com/stoffesvensson/DeepLearning-ThesisWork-Convolutional/"
        "raw/master/webscraper/datasetSameAmountPosNeg"
    )

    # Download the dataset
    csv_str = requests.get(url).text.replace("\r", "")
    csv_file = io.StringIO(csv_str)

    # Convert the dataset to a dataframe
    df = pd.read_csv(csv_file, sep=",", usecols=["text", "rating"])  #  type: ignore[no-matching-overload]
    df.columns = ["text", "label"]

    # Strip trailing whitespace
    df.text = df.text.str.strip()

    # Capitalize the first letter of each sentence
    df.text = df.text.str.capitalize()

    # Remove duplicates
    df = df.drop_duplicates().reset_index(drop=True)

    # Create new splits
    full_train_df, val_test_df = train_test_split(
        df, test_size=2048 + 256, random_state=4242, stratify=df.label
    )
    assert isinstance(full_train_df, pd.DataFrame)
    assert isinstance(val_test_df, pd.DataFrame)
    val_df, test_df = train_test_split(
        val_test_df, test_size=2048, random_state=4242, stratify=val_test_df.label
    )
    _, train_df = train_test_split(
        full_train_df, test_size=1024, random_state=4242, stratify=full_train_df.label
    )

    assert isinstance(train_df, pd.DataFrame)
    assert isinstance(val_df, pd.DataFrame)
    assert isinstance(test_df, pd.DataFrame)

    # Reset the index
    train_df = train_df.reset_index(drop=True)
    val_df = val_df.reset_index(drop=True)
    test_df = test_df.reset_index(drop=True)

    # Only work with samples where the document is not very large or small
    # We do it after we have made the splits to ensure that the dataset is minimally
    # affected.
    new_train_df = train_df.copy()
    new_train_df["text_len"] = new_train_df.text.str.len()
    new_train_df = new_train_df.query("text_len >= @MIN_NUM_CHARS_IN_DOCUMENT").query(
        "text_len <= @MAX_NUM_CHARS_IN_DOCUMENT"
    )
    new_val_df = val_df.copy()
    new_val_df["text_len"] = new_val_df.text.str.len()
    new_val_df = new_val_df.query("text_len >= @MIN_NUM_CHARS_IN_DOCUMENT").query(
        "text_len <= @MAX_NUM_CHARS_IN_DOCUMENT"
    )
    new_test_df = test_df.copy()
    new_test_df["text_len"] = new_test_df.text.str.len()
    new_test_df = new_test_df.query("text_len >= @MIN_NUM_CHARS_IN_DOCUMENT").query(
        "text_len <= @MAX_NUM_CHARS_IN_DOCUMENT"
    )

    df["text_len"] = df.text.str.len()

    # Add train samples back in, if we removed any
    num_new_train_samples = len(train_df) - len(new_train_df)
    if num_new_train_samples > 0:
        new_samples = (
            df.query("text not in @train_df.text")
            .query("text not in @val_df.text")
            .query("text not in @test_df.text")
            .query("text_len >= @MIN_NUM_CHARS_IN_DOCUMENT")
            .query("text_len <= @MAX_NUM_CHARS_IN_DOCUMENT")
            .sample(num_new_train_samples, random_state=4242)
            .drop(columns=["text_len"])
        )
        train_df = (
            pd.concat([new_train_df, new_samples], ignore_index=True)
            .drop(columns=["text_len"])
            .reset_index(drop=True)
        )

    # Add validation samples back in, if we removed any
    num_new_val_samples = len(val_df) - len(new_val_df)
    if num_new_val_samples > 0:
        new_samples = (
            df.query("text not in @train_df.text")
            .query("text not in @val_df.text")
            .query("text not in @test_df.text")
            .query("text_len >= @MIN_NUM_CHARS_IN_DOCUMENT")
            .query("text_len <= @MAX_NUM_CHARS_IN_DOCUMENT")
            .sample(num_new_val_samples, random_state=4242)
            .drop(columns=["text_len"])
        )
        val_df = (
            pd.concat([new_val_df, new_samples], ignore_index=True)
            .drop(columns=["text_len"])
            .reset_index(drop=True)
        )

    # Add test samples back in, if we removed any
    num_new_test_samples = len(test_df) - len(new_test_df)
    if num_new_test_samples > 0:
        new_samples = (
            df.query("text not in @train_df.text")
            .query("text not in @val_df.text")
            .query("text not in @test_df.text")
            .query("text_len >= @MIN_NUM_CHARS_IN_DOCUMENT")
            .query("text_len <= @MAX_NUM_CHARS_IN_DOCUMENT")
            .sample(num_new_test_samples, random_state=4242)
            .drop(columns=["text_len"])
        )
        test_df = (
            pd.concat([new_test_df, new_samples], ignore_index=True)
            .drop(columns=["text_len"])
            .reset_index(drop=True)
        )

    # Collect datasets in a dataset dictionary
    dataset = DatasetDict(
        {
            "train": Dataset.from_pandas(train_df, split=Split.TRAIN),
            "val": Dataset.from_pandas(val_df, split=Split.VALIDATION),
            "test": Dataset.from_pandas(test_df, split=Split.TEST),
        }
    )

    # Push the dataset to the Hugging Face Hub
    dataset_id = "EuroEval/swerec-mini"
    HfApi().delete_repo(dataset_id, repo_type="dataset", missing_ok=True)
    dataset.push_to_hub(dataset_id, private=True)


if __name__ == "__main__":
    main()



"""All Hungarian dataset configurations used in EuroEval."""

from ..data_models import DatasetConfig
from ..languages import HUNGARIAN
from ..tasks import COMMON_SENSE, KNOW, LA, NER, RC, SENT, SUMM

### Official datasets ###

HUSST_CONFIG = DatasetConfig(
    name="husst",
    pretty_name="HuSST",
    source="EuroEval/husst-mini",
    task=SENT,
    languages=[HUNGARIAN],
)

SCALA_HU_CONFIG = DatasetConfig(
    name="scala-hu",
    pretty_name="ScaLA-hu",
    source="EuroEval/scala-hu",
    task=LA,
    languages=[HUNGARIAN],
)

SZEGED_NER_CONFIG = DatasetConfig(
    name="szeged-ner",
    pretty_name="SzegedNER",
    source="EuroEval/szeged-ner",
    task=NER,
    languages=[HUNGARIAN],
)

MULTI_WIKI_QA_HU_CONFIG = DatasetConfig(
    name="multi-wiki-qa-hu",
    pretty_name="MultiWikiQA-hu",
    source="EuroEval/multi-wiki-qa-hu-mini",
    task=RC,
    languages=[HUNGARIAN],
)

HUNSUM_CONFIG = DatasetConfig(
    name="hunsum",
    pretty_name="HunSum",
    source="EuroEval/hun-sum-mini",
    task=SUMM,
    languages=[HUNGARIAN],
)

MMLU_HU_CONFIG = DatasetConfig(
    name="mmlu-hu",
    pretty_name="MMLU-hu",
    source="EuroEval/mmlu-hu-mini",
    task=KNOW,
    languages=[HUNGARIAN],
)

WINOGRANDE_HU_CONFIG = DatasetConfig(
    name="winogrande-hu",
    pretty_name="Winogrande-hu",
    source="EuroEval/winogrande-hu",
    task=COMMON_SENSE,
    languages=[HUNGARIAN],
    _labels=["a", "b"],
)



"""All Romanian dataset configurations used in EuroEval."""

from ..data_models import DatasetConfig
from ..languages import ROMANIAN
from ..tasks import COMMON_SENSE, KNOW, LA, NER, RC, SENT, SUMM

### Official datasets ###

ROSENT_CONFIG = DatasetConfig(
    name="ro-sent",
    pretty_name="RoSent",
    source="EuroEval/ro-sent-mini",
    task=SENT,
    languages=[ROMANIAN],
    _labels=["positive", "negative"],
)

SCALA_RO_CONFIG = DatasetConfig(
    name="scala-ro",
    pretty_name="ScaLA-ro",
    source="EuroEval/scala-ro",
    task=LA,
    languages=[ROMANIAN],
)

RONEC_CONFIG = DatasetConfig(
    name="ronec",
    pretty_name="RoNEC",
    source="EuroEval/ronec-mini",
    task=NER,
    languages=[ROMANIAN],
)

MULTI_WIKI_QA_RO_CONFIG = DatasetConfig(
    name="multi-wiki-qa-ro",
    pretty_name="MultiWikiQA-ro",
    source="EuroEval/multi-wiki-qa-ro-mini",
    task=RC,
    languages=[ROMANIAN],
)

SUMO_RO_CONFIG = DatasetConfig(
    name="sumo-ro",
    pretty_name="SumO-Ro",
    source="EuroEval/sumo-ro-mini",
    task=SUMM,
    languages=[ROMANIAN],
)

GLOBAL_MMLU_RO_CONFIG = DatasetConfig(
    name="global-mmlu-ro",
    pretty_name="GlobalMMLU-ro",
    source="EuroEval/global-mmlu-ro-mini",
    task=KNOW,
    languages=[ROMANIAN],
)

WINOGRANDE_RO_CONFIG = DatasetConfig(
    name="winogrande-ro",
    pretty_name="Winogrande-ro",
    source="EuroEval/winogrande-ro",
    task=COMMON_SENSE,
    languages=[ROMANIAN],
    _labels=["a", "b"],
)



"""All Bosnian dataset configurations used in EuroEval."""

from ..data_models import DatasetConfig
from ..languages import BOSNIAN
from ..tasks import NER, RC, SENT, SUMM

### Official datasets ###

MMS_BS_CONFIG = DatasetConfig(
    name="mms-bs",
    pretty_name="MMS-bs",
    source="EuroEval/mms-bs-mini",
    task=SENT,
    languages=[BOSNIAN],
)

WIKIANN_BS_CONFIG = DatasetConfig(
    name="wikiann-bs",
    pretty_name="WikiANN-bs",
    source="EuroEval/wikiann-bs-mini",
    task=NER,
    languages=[BOSNIAN],
)

MULTI_WIKI_QA_BS_CONFIG = DatasetConfig(
    name="multi-wiki-qa-bs",
    pretty_name="MultiWikiQA-bs",
    source="EuroEval/multi-wiki-qa-bs-mini",
    task=RC,
    languages=[BOSNIAN],
)

LR_SUM_BS_CONFIG = DatasetConfig(
    name="lr-sum-bs",
    pretty_name="LRSum-bs",
    source="EuroEval/lr-sum-bs-mini",
    task=SUMM,
    languages=[BOSNIAN],
)



# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==3.5.0",
#     "huggingface-hub==0.24.0",
#     "pandas==2.2.0",
#     "requests==2.32.3",
# ]
# ///

"""Create the SST5-mini sentiment dataset and upload it to the HF Hub."""

import pandas as pd
from datasets import Dataset, DatasetDict, Split, load_dataset
from huggingface_hub import HfApi

from .constants import MAX_NUM_CHARS_IN_DOCUMENT, MIN_NUM_CHARS_IN_DOCUMENT  # noqa


def main() -> None:
    """Create the SST5-mini sentiment dataset and upload it to the HF Hub."""
    # Define the base download URL
    repo_id = "benjaminvdb/dbrd"

    # Download the dataset
    dataset = load_dataset(path=repo_id, token=True)
    assert isinstance(dataset, DatasetDict)

    # Convert the dataset to a dataframe
    train_df = dataset["train"].to_pandas()
    test_df = dataset["test"].to_pandas()
    assert isinstance(train_df, pd.DataFrame)
    assert isinstance(test_df, pd.DataFrame)

    # Rename the label column to strings
    label_mapping = {0: "negative", 1: "positive"}
    train_df["label"] = train_df["label"].apply(label_mapping.get)
    test_df["label"] = test_df["label"].apply(label_mapping.get)

    # Remove samples ending with "…"
    train_df = train_df[train_df.text.map(lambda x: not x.endswith("…"))]
    test_df = test_df[test_df.text.map(lambda x: not x.endswith("…"))]

    # We impose stricter maximum sizes of documents, as many of them are too long
    max_num_chars = min(MAX_NUM_CHARS_IN_DOCUMENT, 3_500)  # noqa: F841

    # Only work with samples where the document is not very large or small
    train_df = train_df.copy()
    train_df["text_len"] = train_df.text.str.len()
    train_df = train_df.query("text_len >= @MIN_NUM_CHARS_IN_DOCUMENT").query(
        "text_len <= @max_num_chars"
    )
    test_df = test_df.copy()
    test_df["text_len"] = test_df.text.str.len()
    test_df = test_df.query("text_len >= @MIN_NUM_CHARS_IN_DOCUMENT").query(
        "text_len <= @max_num_chars"
    )

    # Create validation split
    val_size = 256
    val_df = train_df.sample(n=val_size, random_state=4242)

    # Create train split
    train_size = 1024
    train_df = train_df[~train_df.index.isin(val_df.index)].sample(
        n=train_size, random_state=4242
    )
    assert isinstance(train_df, pd.DataFrame)

    # Create test split
    test_size = 2048
    test_df = test_df.sample(n=test_size, random_state=4242)

    # Reset the index
    train_df = train_df.reset_index(drop=True)
    val_df = val_df.reset_index(drop=True)
    test_df = test_df.reset_index(drop=True)

    # Collect datasets in a dataset dictionary
    dataset = DatasetDict(
        {
            "train": Dataset.from_pandas(train_df, split=Split.TRAIN),
            "val": Dataset.from_pandas(val_df, split=Split.VALIDATION),
            "test": Dataset.from_pandas(test_df, split=Split.TEST),
        }
    )

    # Create dataset ID
    dataset_id = "EuroEval/dbrd-mini"

    # Remove the dataset from Hugging Face Hub if it already exists
    HfApi().delete_repo(dataset_id, repo_type="dataset", missing_ok=True)

    # Push the dataset to the Hugging Face Hub
    dataset.push_to_hub(dataset_id, private=True)


if __name__ == "__main__":
    main()



# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==3.5.0",
#     "huggingface-hub==0.24.0",
#     "pandas==2.2.0",
#     "requests==2.32.3",
# ]
# ///

"""Create the SumO-Ro summarisation dataset."""

import pandas as pd
from datasets import Dataset, DatasetDict, Split, load_dataset
from huggingface_hub import HfApi

from .constants import MAX_NUM_CHARS_IN_ARTICLE, MIN_NUM_CHARS_IN_ARTICLE


def main() -> None:
    """Create the SumO-Ro summarisation dataset and upload to HF Hub."""
    dataset_id = "Gabrielaaaaaa/SumO-Ro"

    # Load the full dataset
    dataset = load_dataset(dataset_id)
    assert isinstance(dataset, DatasetDict)

    # Convert to pandas for processing
    train_df = dataset["train"].to_pandas()
    val_df = dataset["validation"].to_pandas()
    test_df = dataset["test"].to_pandas()
    assert isinstance(train_df, pd.DataFrame)
    assert isinstance(val_df, pd.DataFrame)
    assert isinstance(test_df, pd.DataFrame)

    train_df = process_df(df=train_df)
    val_df = process_df(df=val_df)
    test_df = process_df(df=test_df)

    train_df_final, val_df_final, test_df_final = make_splits(
        train_df=train_df, val_df=val_df, test_df=test_df
    )

    # Collect datasets in a dataset dictionary
    mini_dataset = DatasetDict(
        {
            "train": Dataset.from_pandas(train_df_final, split=Split.TRAIN),
            "val": Dataset.from_pandas(val_df_final, split=Split.VALIDATION),
            "test": Dataset.from_pandas(test_df_final, split=Split.TEST),
        }
    )

    # Create dataset ID
    mini_dataset_id = "EuroEval/sumo-ro-mini"

    # Remove the dataset from Hugging Face Hub if it already exists
    HfApi().delete_repo(mini_dataset_id, repo_type="dataset", missing_ok=True)

    # Push the dataset to the Hugging Face Hub
    mini_dataset.push_to_hub(mini_dataset_id, private=True)


def process_df(df: pd.DataFrame) -> pd.DataFrame:
    """Process the dataframe to have the text and target_text columns.

    Args:
        df: The dataframe to process.

    Returns:
        The processed dataframe.
    """
    # Only keep rows where column "input_text" starts with <SUMMARY>
    df = df.loc[df["input_text"].str.startswith("<SUMMARY>")]

    # Make text column which is the input_text column without the <SUMMARY> prefix
    df["text"] = df["input_text"].str.replace("<SUMMARY>", "", regex=False)

    # Strip leading and trailing whitespace
    df["text"] = df["text"].str.strip()

    # Only keep rows where the text is not very large or small
    lengths = df.text.str.len()
    lower_bound = MIN_NUM_CHARS_IN_ARTICLE
    upper_bound = MAX_NUM_CHARS_IN_ARTICLE
    df = df.loc[lengths.between(lower_bound, upper_bound)]

    # Only keep the text and target_text columns
    keep_columns = ["text", "target_text"]
    df = df.loc[keep_columns]
    return df


def make_splits(
    train_df: pd.DataFrame, val_df: pd.DataFrame, test_df: pd.DataFrame
) -> tuple[pd.DataFrame, pd.DataFrame, pd.DataFrame]:
    """Make splits for the dataset.

    Args:
        train_df: The training dataframe.
        val_df: The validation dataframe.
        test_df: The test dataframe.

    Returns:
        The final training, validation, and test dataframes.
    """
    # Split sizes
    train_size = 1024
    val_size = 256
    test_size = 2048

    # Create train split
    train_df_final = train_df.sample(n=train_size, random_state=4242)

    # Create validation split
    val_df_final = val_df.sample(n=val_size, random_state=4242)
    remaining_val_df = val_df[~val_df.index.isin(val_df_final.index)]

    # Create test split
    n_missing_test_samples = test_size - len(test_df)
    additional_test_samples = remaining_val_df.sample(
        n=n_missing_test_samples, random_state=4242
    )
    test_df_final = pd.concat([test_df, additional_test_samples], ignore_index=True)

    # Reset indices
    train_df_final = train_df_final.reset_index(drop=True)
    val_df_final = val_df_final.reset_index(drop=True)
    test_df_final = test_df_final.reset_index(drop=True)

    assert isinstance(train_df_final, pd.DataFrame)
    assert isinstance(val_df_final, pd.DataFrame)
    assert isinstance(test_df_final, pd.DataFrame)

    return train_df_final, val_df_final, test_df_final


if __name__ == "__main__":
    main()



# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==3.5.0",
#     "huggingface-hub==0.24.0",
#     "pandas==2.2.0",
#     "requests==2.32.3",
# ]
# ///

"""Create the Catalan GuiaCat sentiment dataset and upload to HF Hub."""

import pandas as pd
from datasets import Dataset, DatasetDict, load_dataset
from huggingface_hub import HfApi


def main() -> None:
    """Create the Catalan GuiaCat sentiment dataset and upload to HF Hub."""
    # Load source dataset
    repo_id = "projecte-aina/GuiaCat"
    dataset = load_dataset(repo_id)
    assert isinstance(dataset, DatasetDict)
    train_df = dataset["train"].to_pandas()
    val_df = dataset["validation"].to_pandas()
    test_df = dataset["test"].to_pandas()
    assert isinstance(train_df, pd.DataFrame)
    assert isinstance(val_df, pd.DataFrame)
    assert isinstance(test_df, pd.DataFrame)

    # Process data
    train_df = process(df=train_df)
    val_df = process(df=val_df)
    test_df = process(df=test_df)

    # Make splits
    train_df_final, val_df_final, test_df_final = make_splits(
        train_df=train_df, val_df=val_df, test_df=test_df
    )

    # Create dataset dictionary
    dataset_dict = DatasetDict(
        {
            "train": Dataset.from_pandas(train_df_final),
            "val": Dataset.from_pandas(val_df_final),
            "test": Dataset.from_pandas(test_df_final),
        }
    )

    # Push to HF Hub
    dataset_id = "EuroEval/guia-cat-mini"
    HfApi().delete_repo(dataset_id, repo_type="dataset", missing_ok=True)
    dataset_dict.push_to_hub(dataset_id, private=True)


def process(df: pd.DataFrame) -> pd.DataFrame:
    """Process the dataframe.

    Args:
        df: The dataframe to process.

    Returns:
        The processed dataframe.
    """
    # Map labels
    label_mapping = {
        "molt bo": "positive",
        "bo": "positive",
        "regular": "neutral",
        "dolent": "negative",
        "molt dolent": "negative",
    }
    df["label"] = df["label"].map(lambda x: label_mapping[x])

    # Keep only text and label columns
    keep_columns = ["text", "label"]
    df = df.loc[keep_columns]
    return df


def make_splits(
    train_df: pd.DataFrame, val_df: pd.DataFrame, test_df: pd.DataFrame
) -> tuple[pd.DataFrame, pd.DataFrame, pd.DataFrame]:
    """Make dataset splits.

    Args:
        train_df: The training dataframe.
        val_df: The validation dataframe.
        test_df: The test dataframe.

    Returns:
        The final training, validation, and test dataframes.
    """
    # Split sizes
    train_size = 1024
    val_size = 256
    test_size = 2048

    # Train split
    train_df_final = train_df.sample(n=train_size, random_state=42)
    train_df_remaining = train_df[~train_df.index.isin(train_df_final.index)]

    # Validation split
    val_df_final = val_df.sample(n=val_size, random_state=42)

    # Test split
    n_missing_test_samples = test_size - len(test_df)
    test_df_additional = train_df_remaining.sample(
        n=n_missing_test_samples, random_state=42
    )
    test_df_final = pd.concat([test_df, test_df_additional], ignore_index=True)

    # Reset the index
    train_df_final = train_df_final.reset_index(drop=True)
    val_df_final = val_df_final.reset_index(drop=True)
    test_df_final = test_df_final.reset_index(drop=True)

    assert isinstance(train_df_final, pd.DataFrame)
    assert isinstance(val_df_final, pd.DataFrame)
    assert isinstance(test_df_final, pd.DataFrame)

    return train_df_final, val_df_final, test_df_final


if __name__ == "__main__":
    main()



"""All Slovak dataset configurations used in EuroEval."""

from ..data_models import DatasetConfig
from ..languages import SLOVAK
from ..tasks import COMMON_SENSE, KNOW, LA, NER, RC, SENT

### Official datasets ###

CSFD_SENTIMENT_SK_CONFIG = DatasetConfig(
    name="csfd-sentiment-sk",
    pretty_name="CSFD Sentiment SK",
    source="EuroEval/csfd-sentiment-sk-mini",
    task=SENT,
    languages=[SLOVAK],
)

SCALA_SK_CONFIG = DatasetConfig(
    name="scala-sk",
    pretty_name="ScaLA-sk",
    source="EuroEval/scala-sk",
    task=LA,
    languages=[SLOVAK],
)

UNER_SK_CONFIG = DatasetConfig(
    name="uner-sk",
    pretty_name="UNER-sk",
    source="EuroEval/uner-sk-mini",
    task=NER,
    languages=[SLOVAK],
)

MULTI_WIKI_QA_SK_CONFIG = DatasetConfig(
    name="multi-wiki-qa-sk",
    pretty_name="MultiWikiQA-sk",
    source="EuroEval/multi-wiki-qa-sk-mini",
    task=RC,
    languages=[SLOVAK],
)

MMLU_SK_CONFIG = DatasetConfig(
    name="mmlu-sk",
    pretty_name="MMLU-sk",
    source="EuroEval/mmlu-sk-mini",
    task=KNOW,
    languages=[SLOVAK],
)

WINOGRANDE_SK_CONFIG = DatasetConfig(
    name="winogrande-sk",
    pretty_name="Winogrande-sk",
    source="EuroEval/winogrande-sk",
    task=COMMON_SENSE,
    languages=[SLOVAK],
)



# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==3.5.0",
#     "huggingface-hub==0.24.0",
#     "pandas==2.2.0",
#     "requests==2.32.3",
# ]
# ///

"""Create the elNER dataset and upload it to the HF Hub."""

import io
import tarfile
from collections import defaultdict

import pandas as pd
import requests
from datasets import Split
from datasets.arrow_dataset import Dataset
from datasets.dataset_dict import DatasetDict
from huggingface_hub.hf_api import HfApi


def main() -> None:
    """Create the elNER dataset and uploads it to the HF Hub."""
    # Define the download URL for the tar.gz file
    url = "https://github.com/nmpartzio/elNER/raw/master/dataset/elNER18/IOB2.tar.gz"

    # Download the dataset
    response = requests.get(url)
    response.raise_for_status()

    # Extract the tar.gz file
    tar_file = tarfile.open(fileobj=io.BytesIO(response.content), mode="r:gz")

    data = read_files_from_tar(tar_file=tar_file)

    dfs = parse_iob2(data=data)

    # Create splits
    train_size = 1024
    val_size = 256
    test_size = 2048

    val_df = dfs["val"].sample(n=val_size, random_state=4242)
    test_df = dfs["test"].sample(n=test_size, random_state=4242)
    train_df = dfs["train"].sample(n=train_size, random_state=4242)

    # Rename ner_tags to labels for consistency
    train_df = train_df.rename(columns={"ner_tags": "labels"}).reset_index(drop=True)
    val_df = val_df.rename(columns={"ner_tags": "labels"}).reset_index(drop=True)
    test_df = test_df.rename(columns={"ner_tags": "labels"}).reset_index(drop=True)

    # Convert the NER tags to standard format
    ner_conversion_dict = {
        # Keep O as is
        "O": "O",
        # Person entities
        "B-PERSON": "B-PER",
        "I-PERSON": "I-PER",
        # Organization entities
        "B-ORG": "B-ORG",
        "I-ORG": "I-ORG",
        # Location entities (including GPE and FAC)
        "B-LOC": "B-LOC",
        "I-LOC": "I-LOC",
        "B-GPE": "B-LOC",  # Geopolitical entity
        "I-GPE": "I-LOC",
        "B-FAC": "B-LOC",  # Facility
        "I-FAC": "I-LOC",
        # Miscellaneous entities
        "B-NORP": "B-MISC",  # Nationality, religion, political group
        "I-NORP": "I-MISC",
        "B-DATE": "O",
        "I-DATE": "O",
        "B-TIME": "O",
        "I-TIME": "O",
        "B-QUANTITY": "O",
        "I-QUANTITY": "O",
        "B-CARDINAL": "O",
        "I-CARDINAL": "O",
        "B-ORDINAL": "O",
        "B-PERCENT": "O",
        "B-MONEY": "O",
        "I-MONEY": "O",
        "B-EVENT": "B-MISC",
        "I-EVENT": "I-MISC",
        "B-WORK_OF_ART": "B-MISC",
        "I-WORK_OF_ART": "I-MISC",
        "B-PRODUCT": "B-MISC",
        "I-PRODUCT": "I-MISC",
        "B-LAW": "O",
        "I-LAW": "O",
        "B-LANGUAGE": "B-MISC",
    }

    # Apply the conversion to the labels
    train_df["labels"] = train_df["labels"].apply(
        lambda tags: [ner_conversion_dict.get(tag, "O") for tag in tags]
    )
    val_df["labels"] = val_df["labels"].apply(
        lambda tags: [ner_conversion_dict.get(tag, "O") for tag in tags]
    )
    test_df["labels"] = test_df["labels"].apply(
        lambda tags: [ner_conversion_dict.get(tag, "O") for tag in tags]
    )

    # Collect datasets in a dataset dictionary
    dataset = DatasetDict(
        {
            "train": Dataset.from_pandas(train_df, split=Split.TRAIN),
            "val": Dataset.from_pandas(val_df, split=Split.VALIDATION),
            "test": Dataset.from_pandas(test_df, split=Split.TEST),
        }
    )

    # Create dataset ID
    dataset_id = "EuroEval/elner-mini"

    # Remove the dataset from Hugging Face Hub if it already exists
    HfApi().delete_repo(dataset_id, repo_type="dataset", missing_ok=True)

    # Push the dataset to the Hugging Face Hub
    dataset.push_to_hub(dataset_id, private=True)


def read_files_from_tar(tar_file: tarfile.TarFile) -> dict[str, list[str]]:
    """Read the files from the tar archive.

    Args:
        tar_file: The tar archive to read the files from.

    Returns:
        A dictionary with the split names as keys and the lines as values.
    """
    # Read the files from the tar archive
    # The structure needs to be checked - typically there are train/test/dev files
    file_members = tar_file.getmembers()

    # Extract and parse the IOB2 files
    # This is a placeholder - we need to see the actual structure first
    data = {}
    for member in file_members:
        if member.isfile() and not member.name.startswith("."):
            content = tar_file.extractfile(member)
            if content:
                text = content.read().decode("utf-8")
                # Determine split name from filename
                if "train" in member.name.lower():
                    split_name = "train"
                elif "test" in member.name.lower():
                    split_name = "test"
                elif "dev" in member.name.lower() or "val" in member.name.lower():
                    split_name = "val"
                else:
                    split_name = member.name

                data[split_name] = text.split("\n")

    return data


def parse_iob2(data: dict[str, list[str]]) -> dict[str, pd.DataFrame]:
    """Parse the IOB2 format.

    Args:
        data: A dictionary with the split names as keys and the lines as values.

    Returns:
        A dictionary with the split names as keys and the dataframes as values.
    """
    # Parse IOB2 format
    dfs = {}
    for split_name, lines in data.items():
        records = []
        data_dict: dict[str, list[str]] = defaultdict(list)

        for line in lines:
            line = line.strip()

            # Empty line indicates end of sentence
            if not line:
                if len(data_dict["tokens"]) > 0:
                    # Join tokens to create text
                    text = " ".join(data_dict["tokens"])
                    merged_data_dict = {
                        "tokens": data_dict["tokens"],
                        "ner_tags": data_dict["ner_tags"],
                        "text": text,
                    }
                    records.append(merged_data_dict)
                data_dict = defaultdict(list)
            else:
                # Parse token and tag (assuming space or tab separated)
                parts = line.split()
                if len(parts) >= 2:
                    token = parts[0]
                    ner_tag = parts[1]
                    data_dict["tokens"].append(token)
                    data_dict["ner_tags"].append(ner_tag)

        # Don't forget the last sentence if file doesn't end with empty line
        if len(data_dict["tokens"]) > 0:
            text = " ".join(data_dict["tokens"])
            merged_data_dict = {
                "tokens": data_dict["tokens"],
                "ner_tags": data_dict["ner_tags"],
                "text": text,
            }
            records.append(merged_data_dict)

        df = pd.DataFrame.from_records(records)

        df = df.drop_duplicates(subset=["text"]).reset_index(drop=True)
        df = df.drop(columns=["text"])

        dfs[split_name] = df

    return dfs


if __name__ == "__main__":
    main()



"""Utility functions to be used in other scripts."""

import asyncio
import collections.abc as c
import gc
import importlib
import importlib.metadata
import importlib.util
import logging
import os
import random
import re
import socket
import sys
import typing as t
from pathlib import Path
from types import ModuleType

import demjson3
import huggingface_hub as hf_hub
import numpy as np
import torch
from huggingface_hub.errors import LocalTokenNotFoundError
from requests.exceptions import RequestException

from .caching_utils import cache_arguments
from .constants import LOCAL_MODELS_REQUIRED_FILES, T
from .exceptions import InvalidBenchmark, InvalidModel, NaNValueInModelOutput
from .logging_utils import log, log_once

if t.TYPE_CHECKING:
    from .data_models import ModelIdComponents
    from .types import Predictions


def create_model_cache_dir(cache_dir: str, model_id: str) -> str:
    """Create cache directory for a model.

    Args:
        cache_dir:
            The cache directory.
        model_id:
            The model ID.

    Returns:
        The path to the cache directory.
    """
    # If the model ID is a path, we just use that as the cache dir
    if Path(model_id).is_dir():
        log_once(
            f"Since the model {model_id!r} is a local model, we will use the model "
            "directory directly as the model cache directory.",
            level=logging.DEBUG,
        )
        return model_id

    # Otherwise, we create a cache dir based on the model ID
    model_cache_dir = Path(
        cache_dir, "model_cache", model_id.replace("/", "--")
    ).as_posix()
    log_once(
        f"Using the model cache directory {model_cache_dir!r} for the model "
        f"{model_id!r}.",
        level=logging.DEBUG,
    )
    return model_cache_dir


def resolve_model_path(download_dir: str) -> str:
    """Resolve the path to the directory containing the model config files and weights.

    Args:
        download_dir:
            The download directory

    Returns:
        The path to the model.

    Raises:
        InvalidModel:
            If the model path is not valid, or if required files are missing.
    """
    model_path = Path(download_dir)

    # Get the 'path safe' version of the model id, which is the last dir in the path
    model_id_path = model_path.name

    # Hf hub `cache_dir` puts the files in models--`model_id_path`/snapshots
    model_path = model_path / f"models--{model_id_path}" / "snapshots"
    if not model_path.exists():
        raise InvalidModel(
            f"Attempted to load models from the {model_path} directory, "
            "but it does not exist."
        )

    # Get all files in the model path
    found_files = [
        found_file for found_file in model_path.rglob("*") if found_file.is_file()
    ]
    if not found_files:
        raise InvalidModel(f"No model files found at {model_path}")

    # Make sure that there arent multiples of the files found
    if len(found_files) == len(set(found_files)):
        raise InvalidModel(
            f"Found multiple model config files for {model_id_path.strip('models--')}"
            f"at {model_path}"
        )

    # Check that found_files contains at least one of the required files
    found_required_file = next(
        (file for file in found_files if file.name in LOCAL_MODELS_REQUIRED_FILES), None
    )
    if found_required_file is None:
        raise InvalidModel(
            f"At least one of the files {LOCAL_MODELS_REQUIRED_FILES} must be present "
            f"for {model_id_path.strip('models--')} at {model_path}"
        )
    model_path = found_required_file.parent

    # As a precaution we also check that all of the files are in the same directory
    # if not we create a new dir with symlinks to all of the files from all snapshots
    # this is especially useful for vllm where we can only specify one folder and e.g.,
    # the safetensors version of the weights was added in an unmerged PR
    if not all(
        [found_file.parent == found_files[0].parent for found_file in found_files]
    ):
        new_model_path = model_path.parent / "model_files"
        new_model_path.mkdir(exist_ok=True)
        for found_file in found_files:
            Path(new_model_path / found_file.name).symlink_to(found_file)
        model_path = new_model_path

    return str(model_path)


def clear_memory() -> None:
    """Clears the memory of unused items."""
    for gc_generation in range(3):
        gc.collect(generation=gc_generation)
    if torch.cuda.is_available():
        torch.cuda.empty_cache()
    if torch.backends.mps.is_available():
        torch.mps.empty_cache()


def enforce_reproducibility(seed: int = 4242) -> np.random.Generator:
    """Ensures reproducibility of experiments.

    Args:
        seed:
            Seed for the random number generator.
    """
    random.seed(seed)
    np.random.seed(seed)
    rng = np.random.default_rng(seed)
    torch.manual_seed(seed)
    torch.cuda.manual_seed_all(seed)
    os.environ["CUDA_LAUNCH_BLOCKING"] = "1"
    os.environ["CUBLAS_WORKSPACE_CONFIG"] = ":4096:8"
    torch.backends.cudnn.benchmark = False
    torch.backends.cudnn.deterministic = True
    torch.use_deterministic_algorithms(True, warn_only=True)
    return rng


def get_class_by_name(
    class_name: str | c.Sequence[str], module_name: str
) -> t.Type | None:
    """Get a class by its name.

    Args:
        class_name:
            The name of the class, written in kebab-case. The corresponding class name
            must be the same, but written in PascalCase, and lying in a module with the
            same name, but written in snake_case. If a list of strings is passed, the
            first class that is found is returned.
        module_name:
            The name of the module where the class is located.

    Returns:
        The class. If the class is not found, None is returned.
    """
    if isinstance(class_name, str):
        class_name = [class_name]

    error_messages = list()
    for name in class_name:
        try:
            module = importlib.import_module(name=module_name)
            class_: t.Type = getattr(module, name)
            return class_
        except (ModuleNotFoundError, AttributeError) as e:
            error_messages.append(str(e))

    if error_messages:
        errors = "\n- " + "\n- ".join(error_messages)
        log(
            f"Could not find the class with the name(s) {', '.join(class_name)}. The "
            f"following error messages were raised: {errors}",
            level=logging.DEBUG,
        )

    # If the class could not be found, return None
    return None


def get_min_cuda_compute_capability() -> float | None:
    """Gets the lowest cuda capability.

    Returns:
        Device capability as float, or None if CUDA is not available.
    """
    if not torch.cuda.is_available():
        return None

    device_range = range(torch.cuda.device_count())
    capabilities = map(torch.cuda.get_device_capability, device_range)
    major, minor = min(capabilities)
    return float(f"{major}.{minor}")


@cache_arguments(disable_condition=lambda: hasattr(sys, "_called_from_test"))
def internet_connection_available() -> bool:
    """Checks if internet connection is available by pinging google.com.

    Returns:
        Whether or not internet connection is available.
    """
    internet_available: bool = False

    try:
        s = socket.create_connection(("1.1.1.1", 80))
        s.close()
        internet_available = True
    except OSError:
        pass
    except Exception as e:
        pytest_socket_errors = ["SocketConnectBlockedError", "SocketBlockedError"]
        if type(e).__name__ not in pytest_socket_errors:
            raise e

    return internet_available


def raise_if_model_output_contains_nan_values(model_output: "Predictions") -> None:
    """Raise an exception if the model output contains NaN values.

    Args:
        model_output:
            The model output to check.

    Raises:
        If the model output contains NaN values.
    """
    if isinstance(model_output, np.ndarray):
        if model_output.dtype == np.float32 and np.isnan(model_output).any():
            raise NaNValueInModelOutput()
    elif len(model_output) > 0:
        if isinstance(model_output[0], str):
            if any(x != x for x in model_output):
                raise NaNValueInModelOutput()
        elif len(model_output[0]) > 0:
            if any(x != x for sublist in model_output for x in sublist):
                raise NaNValueInModelOutput()


def scramble(text: str) -> str:
    """Scramble a string in a bijective manner.

    Args:
        text:
            The string to scramble.

    Returns:
        The scrambled string.
    """
    rng = np.random.default_rng(seed=4242)
    permutation = rng.permutation(x=len(text))
    scrambled = "".join(text[i] for i in permutation)
    return scrambled


def unscramble(scrambled_text: str) -> str:
    """Unscramble a string in a bijective manner.

    Args:
        scrambled_text:
            The scrambled string to unscramble.

    Returns:
        The unscrambled string.
    """
    rng = np.random.default_rng(seed=4242)
    permutation = rng.permutation(x=len(scrambled_text))
    inverse_permutation = np.argsort(permutation)
    unscrambled = "".join(scrambled_text[i] for i in inverse_permutation)
    return unscrambled


def get_package_version(package_name: str) -> str | None:
    """Get the version of a package.

    Args:
        package_name:
            The name of the package.

    Returns:
        The version of the package, or None if the package is not installed.
    """
    try:
        return importlib.metadata.version(package_name)
    except importlib.metadata.PackageNotFoundError:
        return None


def safe_run(coroutine: t.Coroutine[t.Any, t.Any, T]) -> T:
    """Run a coroutine, ensuring that the event loop is always closed when we're done.

    Args:
        coroutine:
            The coroutine to run.

    Returns:
        The result of the coroutine.
    """
    try:
        loop = asyncio.get_event_loop()
    except RuntimeError:  # If the current event loop is closed
        loop = asyncio.new_event_loop()
        asyncio.set_event_loop(loop)
    response = loop.run_until_complete(coroutine)
    return response


async def add_semaphore_and_catch_exception(
    coroutine: t.Coroutine[t.Any, t.Any, T], semaphore: asyncio.Semaphore
) -> T | Exception:
    """Run a coroutine with a semaphore.

    Args:
        coroutine:
            The coroutine to run.
        semaphore:
            The semaphore to use.

    Returns:
        The result of the coroutine.
    """
    async with semaphore:
        try:
            return await coroutine
        except Exception as exc:
            return exc


def extract_json_dict_from_string(s: str) -> dict | None:
    """Extract a JSON dictionary from a string.

    Args:
        s:
            The string to extract the JSON dictionary from.

    Returns:
        The extracted JSON dictionary, or None if no JSON dictionary could be found.
    """
    json_regex = r"\{[^{}]*?\}"
    if (json_match := re.search(pattern=json_regex, string=s, flags=re.DOTALL)) is None:
        log(
            "The model output does not contain any JSON dictionary, so cannot parse "
            f"it. Skipping. Here is the output: {s!r}",
            level=logging.DEBUG,
        )
        return None
    json_string = json_match.group()
    try:
        json_output = demjson3.decode(txt=json_string)
    except demjson3.JSONDecodeError:
        log(
            "The model output is not valid JSON, so cannot parse it. Skipping. "
            f"Here is the output: {json_string!r}",
            level=logging.DEBUG,
        )
        return None
    if not isinstance(json_output, dict):
        log(
            "The model output is not a JSON dictionary, so cannot parse "
            f"it. Skipping. Here is the output: {json_string!r}",
            level=logging.DEBUG,
        )
        return None
    elif not all(isinstance(key, str) for key in json_output.keys()):
        log(
            "The model output is not a JSON dictionary with string keys, "
            "so cannot parse it. Skipping. Here is the output: "
            f"{json_string!r}",
            level=logging.DEBUG,
        )
        return None
    return json_output


@cache_arguments()
def get_hf_token(api_key: str | None) -> str | bool:
    """Get the Hugging Face token.

    Args:
        api_key:
            The API key to use as the Hugging Face token. If None, we will try to
            extract it in other ways.

    Returns:
        The Hugging Face token, or True if no token is set but the user is logged in, or
        False if no token is set and the user is not logged in.
    """
    if api_key is not None:
        log_once(
            "Using the Hugging Face API key passed to the function.",
            level=logging.DEBUG,
        )
        return api_key
    elif (token := os.getenv("HUGGINGFACE_API_KEY")) is not None:
        log_once(
            "Using the Hugging Face API key from the environment variable "
            "`HUGGINGFACE_API_KEY`.",
            level=logging.DEBUG,
        )
        return token
    try:
        hf_hub.whoami()
        log_once(
            "No Hugging Face API key was set, but the user is logged in to Hugging "
            "Face, so using the local token.",
            level=logging.DEBUG,
        )
        return True
    except LocalTokenNotFoundError:
        log_once(
            "No Hugging Face API key was set and the user is not logged in to Hugging "
            "Face, so no token will be used.",
            level=logging.DEBUG,
        )
        return False
    except RequestException:
        log_once(
            "No Hugging Face API key was set and the connection to Hugging Face "
            "failed, so no token will be used.",
            level=logging.DEBUG,
        )
        return False


def extract_multiple_choice_labels(
    prompt: str, candidate_labels: c.Sequence[str]
) -> c.Sequence[str]:
    """Extract multiple choice labels from a prompt.

    Args:
        prompt:
            The prompt to extract the labels from.
        candidate_labels:
            The candidate labels to look for in the prompt.

    Returns:
        The extracted labels.
    """
    sample_candidate_labels: list[str] = list()
    for candidate_label in candidate_labels:
        candidate_label_match = re.search(
            pattern=rf"\b{candidate_label}\. ", string=prompt, flags=re.IGNORECASE
        )
        if candidate_label_match is not None:
            sample_candidate_labels.append(candidate_label)
    if not sample_candidate_labels:
        raise InvalidBenchmark(
            "Could not extract any candidate labels from the prompt. Please ensure "
            "that the candidate labels are present in the prompt, each followed by a "
            "dot and a space (e.g., 'a. '). The candidate labels are: "
            f"{', '.join(candidate_labels)}. Here is the prompt: {prompt!r}"
        )
    return sample_candidate_labels


def split_model_id(model_id: str) -> "ModelIdComponents":
    """Split a model ID into its components.

    Args:
        model_id:
            The model ID to split.

    Returns:
        The split model ID.

    Raises:
        If the model ID is not valid.
    """
    # Importing here to avoid circular imports
    from .data_models import ModelIdComponents

    # Attempt to extract the model ID, revision, and param using regex
    model_id_match = re.match(pattern=r"^[^@#]+", string=model_id)
    revision_match = re.search(pattern=r"@([^@#]+)", string=model_id)
    param_match = re.search(pattern=r"#([^@#]+)", string=model_id)

    # If we cannot extract the model ID, raise an error
    if model_id_match is None:
        raise InvalidModel(f"The model ID {model_id!r} is not valid.")
    model_id = model_id_match.group()

    # Extract the revision and param and return the result
    revision = revision_match.group(1) if revision_match is not None else "main"
    param = param_match.group(1) if param_match is not None else None
    return ModelIdComponents(model_id=model_id, revision=revision, param=param)


def load_custom_datasets_module(custom_datasets_file: Path) -> ModuleType | None:
    """Load the custom datasets module if it exists.

    Args:
        custom_datasets_file:
            The path to the custom datasets module.

    Raises:
        RuntimeError:
            If the custom datasets module cannot be loaded.
    """
    if custom_datasets_file.exists():
        spec = importlib.util.spec_from_file_location(
            name="custom_datasets_module", location=str(custom_datasets_file.resolve())
        )
        if spec is None:
            log_once(
                "Could not load the spec for the custom datasets file from "
                f"{custom_datasets_file.resolve()}.",
                level=logging.ERROR,
            )
            return None
        module = importlib.util.module_from_spec(spec=spec)
        if spec.loader is None:
            log_once(
                "Could not load the module for the custom datasets file from "
                f"{custom_datasets_file.resolve()}.",
                level=logging.ERROR,
            )
            return None
        spec.loader.exec_module(module)
        return module
    return None



# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==2.15.0",
#     "huggingface-hub==0.24.0",
#     "openai==1.66.5",
#     "pandas==2.2.0",
#     "python-dotenv==1.0.1",
#     "requests==2.32.3",
# ]
# ///

"""Create the NorGLM NO-multi question answering dataset."""

import ast
import hashlib
import os

import pandas as pd
from datasets import Dataset, DatasetDict, Split, load_dataset
from dotenv import load_dotenv
from huggingface_hub import HfApi
from openai import OpenAI

from .constants import (
    MAX_NUM_CHARS_IN_CONTEXT,
    MAX_NUM_CHARS_IN_QUESTION,
    MIN_NUM_CHARS_IN_CONTEXT,
    MIN_NUM_CHARS_IN_QUESTION,
)

load_dotenv()


def main() -> None:
    """Create the NorGLM NO-multi question answering dataset and upload to HF Hub."""
    dataset_id = "NorGLM/NO-Multi-QA-Sum"
    client = OpenAI(api_key=os.environ["OPENAI_API_KEY"])

    dataset = load_dataset(dataset_id, split="train", token=True)
    assert isinstance(dataset, Dataset)

    dataset = dataset.rename_columns(column_mapping=dict(article="context"))

    df = dataset.to_pandas()
    assert isinstance(df, pd.DataFrame)

    # Drop unneeded index column
    df.drop("Unnamed: 0", inplace=True, axis=1)

    # Drop non-article by index
    df.drop(index=359, inplace=True)

    # Shuffle and drop first duplicate
    df = df.sample(frac=1, random_state=4242).drop_duplicates(subset="context")

    # Reset the index
    df = df.reset_index(drop=True)

    # Convert the question_answer column to a list of tuples
    def qa_to_list(row: str) -> list:
        """Converts a string representation of a list of tuples to an actual list.

        Args:
            row:
                The row to convert

        Returns:
            The row converted to an actual list of tuples
        """
        qa_list = ast.literal_eval(row)
        quest_ans = [(q.strip(), a.strip()) for q, a in qa_list]
        return quest_ans

    df["question_answer"] = df["question_answer"].apply(qa_to_list)

    # split question_answer list [(question, answer) ...] column to question and answer
    # with same context
    df = df.explode("question_answer")
    df[["question", "answer"]] = pd.DataFrame(
        df.question_answer.tolist(), index=df.index
    )
    df.drop(["question_answer", "summary"], inplace=True, axis=1)
    df.reset_index(drop=True, inplace=True)

    # Only work with samples where the context is not very large or small
    lengths = df.context.str.len()
    lower_bound = MIN_NUM_CHARS_IN_CONTEXT
    upper_bound = MAX_NUM_CHARS_IN_CONTEXT
    df = df[lengths.between(lower_bound, upper_bound)]

    # Only work with samples where the question is not very large or small
    lengths = df.question.str.len()
    lower_bound = MIN_NUM_CHARS_IN_QUESTION
    upper_bound = MAX_NUM_CHARS_IN_QUESTION
    df = df[lengths.between(lower_bound, upper_bound)]
    assert isinstance(df, pd.DataFrame)

    def rephrase_answer(question: str, answer: str, context: str) -> str:
        """Rephrase the answer such that it is in the context.

        Args:
            question:
                The question.
            answer:
                The answer.
            context:
                The context.

        Returns:
            The rephrased answer (if the answer is already in the context, it is
            returned as is).
        """
        answer = answer[:-1] if answer.endswith(".") else answer

        if answer.lower() in context.lower():
            return answer

        # Use OpenAI to rephrase the answer
        chat_completion = client.chat.completions.create(
            messages=[
                {
                    "role": "user",
                    "content": (
                        f"Given the following context: '{context}', and the "
                        f"'{question}' please rephrase the answer, so that it matches "
                        "exactly with a phrase from the context in a case-insensitive "
                        "way. E.g. it must be possible to find the rephrased answer "
                        "in the context without any modifications. The rephrased "
                        "answer should be as concise as possible, preferable not "
                        "more than 7 words, and ideally 3 or less words. Now "
                        f"rephrase this answer: '{answer}'"
                    ),
                }
            ],
            model="gpt-4o",
            seed=4242,
            temperature=0,
        )

        rephrased_answer = chat_completion.choices[0].message.content
        return rephrased_answer or answer

    # Use the original answer if it could be found in the context
    df_orig_bool = df[["answer", "context"]].apply(
        lambda row: row["answer"].lower() in row["context"].lower(), axis=1
    )
    df_orig = df[df_orig_bool]

    # For the rest of the answers, we try to rephrase them
    df_no_context = df[~df_orig_bool]

    # Rephrase the answers
    df_no_context.loc[:, "answer"] = df_no_context.apply(
        lambda row: rephrase_answer(row["question"], row["answer"], row["context"]),
        axis=1,
    )
    assert isinstance(df_no_context, pd.DataFrame)

    # Remove non-word start and end characters from answers
    df_no_context.loc[:, "answer"] = df_no_context["answer"].str.replace(
        r"(^\W|\W$)", "", regex=True
    )

    # Only keep the rephrased answers where the answer could be found in the context
    df_in_context = df_no_context[["answer", "context"]].apply(
        lambda row: row["answer"].lower() in row["context"].lower(), axis=1
    )
    df_with_answer = df_no_context[df_in_context]

    # Combine original with the rephrased answers
    cleaned_df = pd.concat([df_orig, df_with_answer])

    cleaned_df = cleaned_df.reset_index(drop=True)

    # Convert to 'answers' style column
    cleaned_df.loc[:, "answers"] = cleaned_df.apply(
        lambda row: {
            "text": [row["answer"]],
            "answer_start": row["context"].lower().index(row["answer"].lower()),
        },
        axis=1,
    )

    # Overwrite the original dataframe with the cleaned version
    df = cleaned_df

    # should not change since we use seed and temp 0 for rephrasing
    assert len(df) == 2406

    # Create validation split
    val_size = 256
    val_df = df.sample(n=val_size, random_state=4242)

    # Create train split
    train_size = 1024
    filtered_df = df[~df.index.isin(val_df.index)]
    assert isinstance(filtered_df, pd.DataFrame)
    train_df = filtered_df.sample(n=train_size, random_state=4242)

    # Create test split, using the remaining samples
    test_df = filtered_df[~filtered_df.index.isin(train_df.index)]

    val_df = val_df.reset_index(drop=True)
    test_df = test_df.reset_index(drop=True)
    train_df = train_df.reset_index(drop=True)

    # Add ID column
    train_df["id"] = [
        hashlib.md5((row.context + row.question).encode("utf-8")).hexdigest()
        for _, row in train_df.iterrows()
    ]
    val_df["id"] = [
        hashlib.md5((row.context + row.question).encode("utf-8")).hexdigest()
        for _, row in val_df.iterrows()
    ]
    test_df["id"] = [
        hashlib.md5((row.context + row.question).encode("utf-8")).hexdigest()
        for _, row in test_df.iterrows()
    ]

    assert isinstance(train_df, pd.DataFrame)
    assert isinstance(val_df, pd.DataFrame)
    assert isinstance(test_df, pd.DataFrame)

    # Check that the IDs are unique
    assert train_df.id.nunique() == len(train_df)
    assert val_df.id.nunique() == len(val_df)
    assert test_df.id.nunique() == len(test_df)

    # Collect datasets in a dataset dictionary
    dataset = DatasetDict(
        {
            "train": Dataset.from_pandas(train_df, split=Split.TRAIN),
            "val": Dataset.from_pandas(val_df, split=Split.VALIDATION),
            "test": Dataset.from_pandas(test_df, split=Split.TEST),
        }
    )

    # Push the dataset to the Hugging Face Hub
    dataset_id = "EuroEval/norglm-multi-qa"
    HfApi().delete_repo(dataset_id, repo_type="dataset", missing_ok=True)
    dataset.push_to_hub(dataset_id, private=True)


if __name__ == "__main__":
    main()



"""All benchmarks tasks used in EuroEval."""

from . import metrics as m
from .constants import NUM_GENERATION_TOKENS_FOR_CLASSIFICATION
from .data_models import Task
from .enums import GenerativeType, ModelType, TaskGroup
from .prompt_templates import (
    CLASSIFICATION_TEMPLATES,
    LA_TEMPLATES,
    MULTIPLE_CHOICE_TEMPLATES,
    NER_TEMPLATES,
    RC_TEMPLATES,
    SENT_TEMPLATES,
    SIMPL_TEMPLATES,
    SUMM_TEMPLATES,
    TOKEN_CLASSIFICATION_TEMPLATES,
)

LA = Task(
    name="linguistic-acceptability",
    task_group=TaskGroup.SEQUENCE_CLASSIFICATION,
    template_dict=LA_TEMPLATES,
    metrics=[m.mcc_metric, m.macro_f1_metric],
    default_num_few_shot_examples=12,
    default_max_generated_tokens=NUM_GENERATION_TOKENS_FOR_CLASSIFICATION,
    default_labels=["correct", "incorrect"],
    uses_logprobs=True,
)


NER = Task(
    name="named-entity-recognition",
    task_group=TaskGroup.TOKEN_CLASSIFICATION,
    template_dict=NER_TEMPLATES,
    metrics=[m.micro_f1_no_misc_metric, m.micro_f1_metric],
    default_num_few_shot_examples=8,
    default_max_generated_tokens=128,
    default_labels=[
        "o",
        "b-loc",
        "i-loc",
        "b-org",
        "i-org",
        "b-per",
        "i-per",
        "b-misc",
        "i-misc",
    ],
    uses_structured_output=True,
)


RC = Task(
    name="reading-comprehension",
    task_group=TaskGroup.QUESTION_ANSWERING,
    template_dict=RC_TEMPLATES,
    metrics=[m.f1_metric, m.em_metric],
    default_num_few_shot_examples=4,
    default_max_generated_tokens=32,
    default_labels=["start_positions", "end_positions"],
)


SENT = Task(
    name="sentiment-classification",
    task_group=TaskGroup.SEQUENCE_CLASSIFICATION,
    template_dict=SENT_TEMPLATES,
    metrics=[m.mcc_metric, m.macro_f1_metric],
    default_num_few_shot_examples=12,
    default_max_generated_tokens=NUM_GENERATION_TOKENS_FOR_CLASSIFICATION,
    default_labels=["positive", "neutral", "negative"],
    uses_logprobs=True,
)

SIMPL = Task(
    name="simplification",
    task_group=TaskGroup.TEXT_TO_TEXT,
    template_dict=SIMPL_TEMPLATES,
    metrics=[m.meteor_metric, m.sari_metric],
    default_num_few_shot_examples=3,
    default_max_generated_tokens=256,
    default_labels=[],
    default_allowed_model_types=[ModelType.GENERATIVE],
)

SUMM = Task(
    name="summarization",
    task_group=TaskGroup.TEXT_TO_TEXT,
    template_dict=SUMM_TEMPLATES,
    metrics=[m.bert_score_metric, m.rouge_l_metric],
    default_num_few_shot_examples=1,
    default_max_generated_tokens=256,
    default_labels=[],
    default_allowed_model_types=[ModelType.GENERATIVE],
)


KNOW = Task(
    name="knowledge",
    task_group=TaskGroup.MULTIPLE_CHOICE_CLASSIFICATION,
    template_dict=MULTIPLE_CHOICE_TEMPLATES,
    metrics=[m.mcc_metric, m.accuracy_metric],
    default_num_few_shot_examples=5,
    default_max_generated_tokens=NUM_GENERATION_TOKENS_FOR_CLASSIFICATION,
    default_labels=["a", "b", "c", "d"],
    default_allowed_model_types=[ModelType.GENERATIVE],
    uses_logprobs=True,
)


MCRC = Task(
    name="multiple-choice-reading-comprehension",
    task_group=TaskGroup.MULTIPLE_CHOICE_CLASSIFICATION,
    template_dict=MULTIPLE_CHOICE_TEMPLATES,
    metrics=[m.mcc_metric, m.accuracy_metric],
    default_num_few_shot_examples=5,
    default_max_generated_tokens=NUM_GENERATION_TOKENS_FOR_CLASSIFICATION,
    default_labels=["a", "b", "c", "d"],
    default_allowed_model_types=[ModelType.GENERATIVE],
    uses_logprobs=True,
)


COMMON_SENSE = Task(
    name="common-sense-reasoning",
    task_group=TaskGroup.MULTIPLE_CHOICE_CLASSIFICATION,
    template_dict=MULTIPLE_CHOICE_TEMPLATES,
    metrics=[m.mcc_metric, m.accuracy_metric],
    default_num_few_shot_examples=5,
    default_max_generated_tokens=NUM_GENERATION_TOKENS_FOR_CLASSIFICATION,
    default_labels=["a", "b", "c", "d"],
    default_allowed_model_types=[ModelType.GENERATIVE],
    uses_logprobs=True,
)


EUROPEAN_VALUES = Task(
    name="european-values",
    task_group=TaskGroup.MULTIPLE_CHOICE_CLASSIFICATION,
    template_dict=MULTIPLE_CHOICE_TEMPLATES,
    metrics=[m.european_values_metric],
    default_num_few_shot_examples=0,
    default_max_generated_tokens=NUM_GENERATION_TOKENS_FOR_CLASSIFICATION,
    default_labels=["a", "b", "c", "d", "e", "f", "g", "h", "i", "j", "k"],
    default_allowed_model_types=[ModelType.GENERATIVE],
    default_allowed_generative_types=[
        GenerativeType.INSTRUCTION_TUNED,
        GenerativeType.REASONING,
    ],
    requires_zero_shot=True,
    uses_logprobs=True,
    default_allow_invalid_model_outputs=False,
)


MCSTEREO = Task(
    name="multiple-choice-stereotype-bias",
    task_group=TaskGroup.MULTIPLE_CHOICE_CLASSIFICATION,
    template_dict=MULTIPLE_CHOICE_TEMPLATES,
    metrics=[
        m.bias_adjusted_accuracy_ambig_metric,
        m.bias_ambig_metric,
        m.accuracy_ambig_metric,
    ],
    default_num_few_shot_examples=0,
    default_max_generated_tokens=NUM_GENERATION_TOKENS_FOR_CLASSIFICATION,
    default_labels=["a", "b", "c"],
    default_allowed_model_types=[ModelType.GENERATIVE],
    default_allowed_generative_types=[
        GenerativeType.INSTRUCTION_TUNED,
        GenerativeType.REASONING,
    ],
    requires_zero_shot=True,
    uses_logprobs=True,
)


SPEED = Task(
    name="speed",
    task_group=TaskGroup.SPEED,
    template_dict={},
    metrics=[m.speed_metric, m.speed_short_metric],
    default_num_few_shot_examples=0,
    default_max_generated_tokens=5,
    default_labels=[],
)


# Used for custom datasets

TEXT_CLASSIFICATION = Task(
    name="classification",
    task_group=TaskGroup.SEQUENCE_CLASSIFICATION,
    template_dict=CLASSIFICATION_TEMPLATES,
    metrics=[m.mcc_metric, m.macro_f1_metric],
    default_num_few_shot_examples=12,
    default_max_generated_tokens=NUM_GENERATION_TOKENS_FOR_CLASSIFICATION,
    default_labels=None,
    uses_logprobs=True,
)

TOKEN_CLASSIFICATION = Task(
    name="token-classification",
    task_group=TaskGroup.TOKEN_CLASSIFICATION,
    template_dict=TOKEN_CLASSIFICATION_TEMPLATES,
    metrics=[m.micro_f1_metric],
    default_num_few_shot_examples=8,
    default_max_generated_tokens=128,
    default_labels=None,
    uses_structured_output=True,
)

MULTIPLE_CHOICE = Task(
    name="multiple-choice",
    task_group=TaskGroup.MULTIPLE_CHOICE_CLASSIFICATION,
    template_dict=MULTIPLE_CHOICE_TEMPLATES,
    metrics=[m.mcc_metric, m.accuracy_metric],
    default_num_few_shot_examples=5,
    default_max_generated_tokens=NUM_GENERATION_TOKENS_FOR_CLASSIFICATION,
    default_labels=None,
    default_allowed_model_types=[ModelType.GENERATIVE],
    uses_logprobs=True,
)



# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==3.5.0",
#     "huggingface-hub==0.24.0",
#     "pandas==2.2.0",
#     "requests==2.32.3",
# ]
# ///

"""Create the CSFD Sentiment dataset and upload to HF Hub."""

import pandas as pd
from datasets import Dataset, DatasetDict, Split, load_dataset
from huggingface_hub import HfApi


def main() -> None:
    """Create the Czech CSFD sentiment dataset and upload to HF Hub."""
    # Define the dataset repository
    repo_id = "CZLC/csfd_sentiment_balanced"

    # Download the dataset
    dataset = load_dataset(path=repo_id)
    assert isinstance(dataset, DatasetDict)

    # Process each split separately
    processed_splits = {}

    for split_name in ["train", "validation", "test"]:
        # Convert to dataframe
        df = dataset[split_name].to_pandas()
        assert isinstance(df, pd.DataFrame)

        # Rename query -> text
        df.rename(columns={"query": "text"}, inplace=True)

        # Create the label column based on gold index
        # gold: 0 = negativní, 1 = neutrální, 2 = pozitivní
        df["label"] = df["gold"].map(
            lambda x: {0: "negative", 1: "neutral", 2: "positive"}[x]
        )

        # Keep only columns text and label
        df = df[["text", "label"]]

        # Remove duplicates within each split
        df = df.drop_duplicates().reset_index(drop=True)

        # Store processed split
        processed_splits[split_name] = df

    # Make splits
    train_df = processed_splits["train"]
    val_df = processed_splits["validation"]
    test_df = processed_splits["test"]

    test_size = 2048
    val_size = 256
    train_size = 1024

    # Calculate how many additional samples needed for test split
    additional_test_samples_needed = test_size - len(test_df)

    # Take additional samples from training set for test split
    additional_test_samples = train_df.sample(
        n=additional_test_samples_needed, random_state=4242
    )

    # Combine original test with additional samples from train
    final_test_df = pd.concat([test_df, additional_test_samples], ignore_index=True)

    # Remove the additional samples from train set
    remaining_train_df = train_df[~train_df.index.isin(additional_test_samples.index)]

    # Sample final splits
    final_test_df = final_test_df.sample(n=test_size, random_state=4242).reset_index(
        drop=True
    )
    final_val_df = val_df.sample(n=val_size, random_state=4242).reset_index(drop=True)
    final_train_df = remaining_train_df.sample(
        n=train_size, random_state=4242
    ).reset_index(drop=True)

    # Create dataset dictionary with custom splits
    dataset = DatasetDict(
        {
            "train": Dataset.from_pandas(final_train_df, split=Split.TRAIN),
            "val": Dataset.from_pandas(final_val_df, split=Split.VALIDATION),
            "test": Dataset.from_pandas(final_test_df, split=Split.TEST),
        }
    )

    # Push the dataset to the Hugging Face Hub
    dataset_id = "EuroEval/csfd-sentiment-mini"
    HfApi().delete_repo(dataset_id, repo_type="dataset", missing_ok=True)
    dataset.push_to_hub(dataset_id, private=True)


if __name__ == "__main__":
    main()



# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==4.0.0",
#     "huggingface-hub==0.34.3",
#     "pandas==2.3.1",
#     "requests==2.32.4",
#     "tqdm==4.67.1",
#     "euroeval==15.16.0",
# ]
# ///

"""Create the European values dataset and upload it to the HF Hub."""

import logging
from collections import defaultdict

import pandas as pd
from datasets import (
    Dataset,
    DatasetDict,
    Split,
    disable_caching,
    disable_progress_bars,
    load_dataset,
)
from huggingface_hub import HfApi
from tqdm.auto import tqdm

from .constants import CHOICES_MAPPING

logging.basicConfig(format="%(asctime)s ⋅ %(message)s", level=logging.INFO)
logger = logging.getLogger("create_european_values")

LANGUAGES = [
    "da",
    "de",
    "en",
    "es",
    "et",
    "fi",
    "fr",
    "is",
    "it",
    "lv",
    "nl",
    "no",
    "pl",
    "pt",
    "sv",
]

QUESTIONS_TO_INCLUDE = [
    "F025:1",
    "F025:5",
    "A124_09",
    "F025:3",
    "F118",
    "D081",
    "C001_01:1",
    "F122",
    "E025",
    "D059",
    "D054",
    "D078",
    "D026_05",
    "E069_01",
    "C041",
    "E003:4",
    "E116",
    "G007_36_B",
    "G007_35_B",
    "E228",
    "E001:2",
    "E265_08",
    "E114",
    "E265_01",
    "C039",
    "E233",
    "E233B",
    "G062",
    "E028",
    "E265_07",
    "E265_06",
    "E265_02",
    "A080_01",
    "E069_02",
    "A080_02",
    "G052",
    "E037",
    "A072",
    "G005",
    "G063",
    "A068",
    "A078",
    "A079",
    "E036",
    "A003",
    "G257",
    "D001_B",
    "F025:8",
    "F025:7",
    "E264:4",
    "A009",
    "E001:4",
    "F025:4",
]


def main() -> None:
    """Create the European values dataset and upload it to the HF Hub."""
    disable_progress_bars()
    disable_caching()

    api = HfApi()
    vanilla_dataset_id = "EuropeanValuesProject/za7505"
    situational_dataset_id = "EuropeanValuesProject/za7505-situational"
    completions_dataset_id = "EuropeanValuesProject/za7505-completions"

    subset_mapping = {
        "da": "da-dk",
        "no": "no-no",
        "sv": "sv-se",
        "is": "is-is",
        "de": "de-de",
        "nl": "nl-nl",
        "en": "en-raw",
        "fr": "fr-fr",
        "it": "it-it",
        "es": "es-es",
        "pl": "pl-pl",
        "pt": "pt-pt",
        "fi": "fi-fi",
        "et": "et-ee",
    }
    no_yes_mapping = {
        "da": {"0": "Nej", "1": "Ja"},
        "no": {"0": "Nei", "1": "Ja"},
        "sv": {"0": "Nej", "1": "Ja"},
        "is": {"0": "Nei", "1": "Já"},
        "de": {"0": "Nein", "1": "Ja"},
        "nl": {"0": "Nee", "1": "Ja"},
        "en": {"0": "No", "1": "Yes"},
        "fr": {"0": "Non", "1": "Oui"},
        "it": {"0": "No", "1": "Sì"},
        "es": {"0": "No", "1": "Sí"},
        "pl": {"0": "Nie", "1": "Tak"},
        "pt": {"0": "Não", "1": "Sim"},
        "fi": {"0": "Ei", "1": "Kyllä"},
        "et": {"0": "Ei", "1": "Jah"},
    }
    sentence_completion_mapping = {
        "da": "Færdiggør følgende sætning",
        "no": "Fullfør følgende setning",
        "sv": "Fyll i följande mening",
        "is": "Ljúktu við eftirfarandi setningu",
        "de": "Vervollständige den folgenden Satz",
        "nl": "Voltooi de volgende zin",
        "en": "Complete the following sentence",
        "fr": "Complétez la phrase suivante",
        "it": "Completa la seguente frase",
        "es": "Completa la siguiente frase",
        "pl": "Uzupełnij następujące zdanie",
        "pt": "Complete a seguinte frase",
        "fi": "Täydennä seuraava lause",
        "et": "Täiendage järgmine lause",
    }

    for dataset_id, new_dataset_id in zip(
        [vanilla_dataset_id, situational_dataset_id, completions_dataset_id],
        [
            "EuroEval/european-values-{language}",
            "EuroEval/european-values-situational-{language}",
            "EuroEval/european-values-completions-{language}",
        ],
    ):
        question_id: str | None = None
        for language in tqdm(iterable=LANGUAGES, desc="Generating datasets"):
            choices_str = CHOICES_MAPPING[language]
            dataset = load_dataset(
                path=dataset_id,
                name=(
                    subset_mapping[language]
                    if dataset_id == vanilla_dataset_id
                    else language
                ),
                split="train",
            )
            assert isinstance(dataset, Dataset)
            df = dataset.to_pandas()
            assert isinstance(df, pd.DataFrame)
            df.set_index("question_id", inplace=True)
            del dataset

            data_dict: dict[str, list] = defaultdict(list)
            for question_id_with_choice in QUESTIONS_TO_INCLUDE:
                question_id = question_id_with_choice.split(":")[0]
                choice_focus = (
                    question_id_with_choice.split(":", 1)[1]
                    if ":" in question_id_with_choice
                    else ""
                )
                if question_id not in df.index:
                    logger.error(
                        f"Question ID {question_id} not found for the language "
                        f"{language}. Skipping this language."
                    )
                    break

                # Extract the question and choices
                question_data = df.loc[question_id]
                if not isinstance(question_data, pd.DataFrame):
                    question_data = pd.DataFrame([question_data])
                for _, row in question_data.iterrows():
                    # Special case for the completions dataset, where we add ellipses to
                    # the question
                    question = row["question"]
                    if dataset_id == completions_dataset_id:
                        question += "..."

                    choices = {
                        key: value[0].upper() + value[1:]
                        for key, value in row.choices.items()
                        if value is not None
                    }

                    # Binary choices are stated as "selected" and "not selected", which
                    # only makes sense when you're ticking off boxes, so we map them to
                    # (the language equivalent of) "yes" and "no"
                    if (
                        sorted(choices.keys()) == ["0", "1"]
                        and dataset_id == vanilla_dataset_id
                    ):
                        choices = no_yes_mapping[language]

                    # Store the mapping from letters to the choice numerals
                    letters = "abcdefghijklmnopqrstuvwxyz"
                    idx_to_choice: dict[str, str] = {
                        str(idx): choice
                        for idx, choice in zip(
                            range(len(choices)), sorted(choices.keys(), key=int)
                        )
                        if choice is not None
                    }
                    choice_to_letter: dict[str, str] = {
                        choice: letters[int(idx)]
                        for idx, choice in idx_to_choice.items()
                    }

                    # Create the prompt string, joining the question and choices
                    prompt = f"{question}\n{choices_str}:\n" + "\n".join(
                        [
                            f"{choice_to_letter[choice]}. {value}"
                            for choice, value in sorted(
                                choices.items(),
                                key=lambda x: int(x[0]) if x[0].isdigit() else x[0],
                            )
                        ]
                    )

                    # Special case for the completions dataset, where we
                    # remove the "Question: " prefix and instead put "Complete the
                    # following sentence: " at the start
                    if dataset_id == completions_dataset_id:
                        prompt = (
                            sentence_completion_mapping[language]
                            + ": "
                            + prompt.split(": ", 1)[-1].strip()
                        )

                    data_dict["question_id"].append(question_id)
                    data_dict["choice"].append(choice_to_letter.get(choice_focus, ""))
                    data_dict["text"].append(prompt)
                    data_dict["idx_to_choice"].append(idx_to_choice)

            if question_id is not None and question_id not in df.index:
                continue
            new_df = pd.DataFrame(data_dict)

            # Collect dataset in a dataset dictionary
            dataset = DatasetDict(
                {"test": Dataset.from_pandas(new_df, split=Split.TEST)}
            )

            # Push the dataset to the Hugging Face Hub, and replace the existing one, if
            # it exists already
            api.delete_repo(
                new_dataset_id.format(language=language),
                repo_type="dataset",
                missing_ok=True,
            )
            dataset.push_to_hub(new_dataset_id.format(language=language), private=True)


if __name__ == "__main__":
    main()



# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==3.5.0",
#     "huggingface-hub==0.24.0",
#     "pandas==2.2.0",
#     "requests==2.32.3",
#     "scikit-learn<1.6.0",
# ]
# ///

"""Create the MMLU-hr dataset and upload to HF Hub."""

from collections import Counter

import pandas as pd
from datasets import Dataset, DatasetDict, Split, load_dataset
from huggingface_hub import HfApi

from .constants import (
    CHOICES_MAPPING,
    MAX_NUM_CHARS_IN_INSTRUCTION,
    MAX_NUM_CHARS_IN_OPTION,
    MAX_REPETITIONS,
    MIN_NUM_CHARS_IN_INSTRUCTION,
    MIN_NUM_CHARS_IN_OPTION,
)


def main() -> None:
    """Create the MMLU-hr dataset and upload to HF Hub."""
    # Read source dataset
    repo_id = "aialt/MuBench"
    dataset = load_dataset(path=repo_id, name="MMLUDataset_local_template_hr")
    assert isinstance(dataset, DatasetDict)
    df_test = dataset["test"].to_pandas()
    df_validation = dataset["validation"].to_pandas()
    assert isinstance(df_test, pd.DataFrame)
    assert isinstance(df_validation, pd.DataFrame)
    df = pd.concat([df_test, df_validation], ignore_index=True)

    # Extract the question and options from the prompt
    df["instruction"], df["choices"] = zip(*df["prompt"].apply(extract_prompt_details))

    # Convert the labels to single lowercase letters
    df.label = df.label.map(lambda x: "abcd"[x])

    # Double check that there are exactly 4 choices per question
    assert df.choices.apply(len).eq(4).all(), "Not all questions have exactly 4 choices"

    # Split up the choices into separate columns
    df["option_a"] = df.choices.apply(lambda x: x[0])
    df["option_b"] = df.choices.apply(lambda x: x[1])
    df["option_c"] = df.choices.apply(lambda x: x[2])
    df["option_d"] = df.choices.apply(lambda x: x[3])

    # Remove the samples with overly short or long texts
    df = df[
        (df.instruction.str.len() >= MIN_NUM_CHARS_IN_INSTRUCTION)
        & (df.instruction.str.len() <= MAX_NUM_CHARS_IN_INSTRUCTION)
        & (df.option_a.str.len() >= MIN_NUM_CHARS_IN_OPTION)
        & (df.option_a.str.len() <= MAX_NUM_CHARS_IN_OPTION)
        & (df.option_b.str.len() >= MIN_NUM_CHARS_IN_OPTION)
        & (df.option_b.str.len() <= MAX_NUM_CHARS_IN_OPTION)
        & (df.option_c.str.len() >= MIN_NUM_CHARS_IN_OPTION)
        & (df.option_c.str.len() <= MAX_NUM_CHARS_IN_OPTION)
        & (df.option_d.str.len() >= MIN_NUM_CHARS_IN_OPTION)
        & (df.option_d.str.len() <= MAX_NUM_CHARS_IN_OPTION)
    ]

    def is_repetitive(text: str) -> bool:
        """Return True if the text is repetitive."""
        max_repetitions = max(Counter(text.split()).values())
        return max_repetitions > MAX_REPETITIONS

    # Remove overly repetitive samples
    df = df[
        ~df.instruction.apply(is_repetitive)
        & ~df.option_a.apply(is_repetitive)
        & ~df.option_b.apply(is_repetitive)
        & ~df.option_c.apply(is_repetitive)
        & ~df.option_d.apply(is_repetitive)
    ]
    assert isinstance(df, pd.DataFrame)

    # Make a `text` column with all the options in it
    df["text"] = [
        row.instruction.replace("\n", " ").strip() + "\n"
        f"{CHOICES_MAPPING['hr']}:\n"
        "a. " + row.option_a.replace("\n", " ").strip() + "\n"
        "b. " + row.option_b.replace("\n", " ").strip() + "\n"
        "c. " + row.option_c.replace("\n", " ").strip() + "\n"
        "d. " + row.option_d.replace("\n", " ").strip()
        for _, row in df.iterrows()
    ]

    # Make the `label` column case-consistent with the `text` column
    df.label = df.label.str.lower()

    # Only keep the `text` and `label` columns
    df = df[["text", "label"]]

    # Remove duplicates
    df.drop_duplicates(inplace=True)
    df.reset_index(drop=True, inplace=True)

    # Make splits
    val_size = 256
    train_size = 1024
    test_size = 2048

    df = df.sample(frac=1, random_state=4242).reset_index(drop=True)
    train_df = df.iloc[:train_size].reset_index(drop=True)
    val_df = df.iloc[train_size : train_size + val_size].reset_index(drop=True)
    test_df = df.iloc[
        train_size + val_size : train_size + val_size + test_size
    ].reset_index(drop=True)

    # Collect datasets in a dataset dictionary
    dataset = DatasetDict(
        {
            "train": Dataset.from_pandas(train_df, split=Split.TRAIN),
            "val": Dataset.from_pandas(val_df, split=Split.VALIDATION),
            "test": Dataset.from_pandas(test_df, split=Split.TEST),
        }
    )

    # Push the dataset to the Hugging Face Hub
    dataset_id = "EuroEval/mmlu-hr-mini"
    HfApi().delete_repo(dataset_id, repo_type="dataset", missing_ok=True)
    dataset.push_to_hub(dataset_id, private=True)


def extract_prompt_details(prompt: str) -> tuple[str, list[str]]:
    """Extract the question and options from the prompt.

    Args:
        prompt: The prompt to extract the question and options from.

    Returns:
        A tuple of the question and options.
    """
    lines = prompt.split("\n")
    question: str = ""
    options: list[str] = []

    for line in lines:
        line = line.strip()
        if line.startswith("Pitanje:"):
            question = line.replace("Pitanje:", "").strip()
        elif line.startswith("Izbor A:"):
            options.append(line.replace("Izbor A:", "").strip())
        elif line.startswith("Izbor B:"):
            options.append(line.replace("Izbor B:", "").strip())
        elif line.startswith("Izbor C:"):
            options.append(line.replace("Izbor C:", "").strip())
        elif line.startswith("Izbor D:"):
            options.append(line.replace("Izbor D:", "").strip())

    return question, options


if __name__ == "__main__":
    main()



# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==3.5.0",
#     "huggingface-hub==0.24.0",
#     "pandas==2.2.0",
#     "requests==2.32.3",
# ]
# ///

"""Create the DaNE-mini NER dataset and upload it to the HF Hub."""

import re
from collections import defaultdict

import pandas as pd
import requests
from datasets import Split
from datasets.arrow_dataset import Dataset
from datasets.dataset_dict import DatasetDict
from huggingface_hub.hf_api import HfApi


def main() -> None:
    """Create the DaNE-mini NER dataset and uploads it to the HF Hub."""
    # Define download URLs
    # TODO: This is the wrong URL; use the alexandra URL instead
    base_url = (
        "https://github.com/UniversalDependencies/UD_Danish-DDT/raw/master/"
        "da_ddt-ud-{}.conllu"
    )
    train_url = base_url.format("train")
    val_url = base_url.format("dev")
    test_url = base_url.format("test")

    # Download the data
    data = dict(
        train=requests.get(train_url).text.split("\n"),
        val=requests.get(val_url).text.split("\n"),
        test=requests.get(test_url).text.split("\n"),
    )

    # Iterate over the data splits
    dfs = dict()
    for split, lines in data.items():
        # Initialise the records, data dictionary and document
        records = list()
        data_dict: dict[str, list[int | str]] = defaultdict(list)
        doc = ""

        # Iterate over the data for the given split
        for line in lines:
            # If we are at the first line of an entry then extract the document
            if line.startswith("# text = "):
                doc = re.sub("# text = ", "", line)

            # Otherwise, if the line is a comment then ignore it
            elif line.startswith("#"):
                continue

            # Otherwise, if we have reached the end of an entry then store it to the
            # list of records and reset the data dictionary and document
            elif line == "":
                if len(data_dict["tokens"]) > 0:
                    merged_data_dict: dict[str, str | list[int | str]]
                    merged_data_dict = {**data_dict, "doc": doc}
                    records.append(merged_data_dict)
                data_dict = defaultdict(list)
                doc = ""

            # Otherwise we are in the middle of an entry which is not a comment, so
            # we extract the data from the line and store it in the data dictionary
            else:
                data_tup = line.split("\t")
                ner_tag = data_tup[9].replace("name=", "").split("|")[0]
                data_dict["ids"].append(data_tup[0])
                data_dict["tokens"].append(data_tup[1])
                data_dict["ner_tags"].append(ner_tag)

        # Convert the records to a dataframe and store it
        dfs[split] = pd.DataFrame.from_records(records)

    # Merge the dataframes
    df = pd.concat(dfs.values()).reset_index(drop=True)

    # Remove duplicates
    df = df.drop_duplicates().reset_index(drop=True)

    # Create new splits
    val_df = df.sample(n=256, random_state=4242)
    df_filtered = df.loc[~df.index.isin(val_df.index)]
    test_df = df_filtered.sample(n=2048, random_state=4242)
    full_train_df = df_filtered.loc[~df_filtered.index.isin(test_df.index)]
    train_df = full_train_df.sample(n=1024, random_state=4242)

    # Collect datasets in a dataset dictionary
    dataset = DatasetDict(
        train=Dataset.from_pandas(train_df, split=Split.TRAIN),
        val=Dataset.from_pandas(val_df, split=Split.VALIDATION),
        test=Dataset.from_pandas(test_df, split=Split.TEST),
        full_train=Dataset.from_pandas(full_train_df, split="full_train"),  # type: ignore[bad-argument-type]
    )

    # Create dataset ID
    dataset_id = "EuroEval/dane-mini"

    # Remove the dataset from Hugging Face Hub if it already exists
    HfApi().delete_repo(dataset_id, repo_type="dataset", missing_ok=True)

    # Push the dataset to the Hugging Face Hub
    dataset.push_to_hub(dataset_id, private=True)


if __name__ == "__main__":
    main()



# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==4.0.0",
#     "huggingface-hub==0.34.4",
#     "pandas==2.2.0",
#     "requests==2.32.5",
#     "scikit-learn==1.6.1",
# ]
# ///

"""Create the HuSST Hungarian sentiment dataset and upload to HF Hub."""

import pandas as pd
from datasets import Dataset, DatasetDict, load_dataset
from huggingface_hub import HfApi
from sklearn.utils import resample


def main() -> None:
    """Create the HuSST Hungarian sentiment dataset and upload to HF Hub."""
    # Load the dataset
    dataset_id = "NYTK/HuSST"
    dataset = load_dataset(dataset_id)
    assert isinstance(dataset, DatasetDict)
    train_df = dataset["train"].to_pandas()
    val_df = dataset["validation"].to_pandas()
    assert isinstance(train_df, pd.DataFrame)
    assert isinstance(val_df, pd.DataFrame)

    train_df = process(df=train_df)
    val_df = process(df=val_df)

    # Create uniform label distribution
    train_df_uniform = create_uniform_label_distribution(df=train_df)
    val_df_uniform = create_uniform_label_distribution(df=val_df)
    val_df_remaining = val_df[~val_df.index.isin(val_df_uniform.index)]

    # Make splits
    train_size = 1024
    val_size = 256
    test_size = 2048

    train_df_final = train_df_uniform.sample(
        n=train_size, random_state=4242
    ).reset_index(drop=True)
    train_df_remaining = train_df_uniform[
        ~train_df_uniform.index.isin(train_df_final.index)
    ]

    test_df_final = train_df_remaining.sample(
        n=test_size, random_state=4242
    ).reset_index(drop=True)

    n_missing_val_samples = val_size - len(val_df_uniform)
    val_df_additional = val_df_remaining.sample(
        n=n_missing_val_samples, random_state=4242
    )
    val_df_final = pd.concat([val_df_uniform, val_df_additional], ignore_index=True)

    assert isinstance(train_df_final, pd.DataFrame)
    assert isinstance(val_df_final, pd.DataFrame)
    assert isinstance(test_df_final, pd.DataFrame)

    # Create DatasetDict
    dataset_dict = DatasetDict(
        {
            "train": Dataset.from_pandas(train_df_final),
            "val": Dataset.from_pandas(val_df_final),
            "test": Dataset.from_pandas(test_df_final),
        }
    )

    # Push to HF Hub
    dataset_id = "EuroEval/husst-mini"
    HfApi().delete_repo(repo_id=dataset_id, repo_type="dataset", missing_ok=True)
    dataset_dict.push_to_hub(dataset_id, private=True)


def process(df: pd.DataFrame) -> pd.DataFrame:
    """Process the dataframe to ensure required columns and rename.

    Args:
        df: The input dataframe.

    Returns:
        A dataframe with the required columns and renamed.
    """
    renames = {"sentence": "text"}
    df = df.rename(columns=renames)
    keep_columns = ["text", "label"]
    df = df.loc[keep_columns]
    return df


def create_uniform_label_distribution(
    df: pd.DataFrame, random_state: int = 4242
) -> pd.DataFrame:
    """Create a sampled dataset with a uniform label distribution.

    Args:
        df: The input dataframe with a 'label' column.
        random_state: The random state for reproducibility.

    Returns:
        A dataframe with a uniform label distribution.
    """
    # Separate each class
    classes = df["label"].unique()
    class_dfs = [df[df["label"] == label] for label in classes]

    # Find the size of the smallest class
    min_size = min(len(class_df) for class_df in class_dfs)

    # Resample each class to the size of the smallest class
    resampled_dfs = [
        resample(class_df, replace=False, n_samples=min_size, random_state=random_state)
        for class_df in class_dfs
    ]

    # Combine the resampled dataframes (keep original indices!)
    balanced_df = pd.concat(resampled_dfs, ignore_index=False)

    return balanced_df


if __name__ == "__main__":
    main()



"""All German dataset configurations used in EuroEval."""

from ..data_models import DatasetConfig
from ..languages import GERMAN
from ..tasks import COMMON_SENSE, EUROPEAN_VALUES, KNOW, LA, MCRC, NER, RC, SENT, SUMM

### Official datasets ###

SB10K_CONFIG = DatasetConfig(
    name="sb10k",
    pretty_name="SB10K",
    source="EuroEval/sb10k-mini",
    task=SENT,
    languages=[GERMAN],
)

SCALA_DE_CONFIG = DatasetConfig(
    name="scala-de",
    pretty_name="ScaLA-de",
    source="EuroEval/scala-de",
    task=LA,
    languages=[GERMAN],
)

GERMEVAL_CONFIG = DatasetConfig(
    name="germeval",
    pretty_name="GermEval",
    source="EuroEval/germeval-mini",
    task=NER,
    languages=[GERMAN],
)

GERMANQUAD_CONFIG = DatasetConfig(
    name="germanquad",
    pretty_name="GermanQuAD",
    source="EuroEval/germanquad-mini",
    task=RC,
    languages=[GERMAN],
)

MLSUM_DE_CONFIG = DatasetConfig(
    name="mlsum-de",
    pretty_name="MLSUM-de",
    source="EuroEval/mlsum-mini",
    task=SUMM,
    languages=[GERMAN],
)

MMLU_DE_CONFIG = DatasetConfig(
    name="mmlu-de",
    pretty_name="MMLU-de",
    source="EuroEval/mmlu-de-mini",
    task=KNOW,
    languages=[GERMAN],
)

HELLASWAG_DE_CONFIG = DatasetConfig(
    name="hellaswag-de",
    pretty_name="HellaSwag-de",
    source="EuroEval/hellaswag-de-mini",
    task=COMMON_SENSE,
    languages=[GERMAN],
)

VALEU_DE_CONFIG = DatasetConfig(
    name="valeu-de",
    pretty_name="VaLEU-de",
    source="EuroEval/european-values-de",
    task=EUROPEAN_VALUES,
    languages=[GERMAN],
    splits=["test"],
    bootstrap_samples=False,
    _instruction_prompt="{text}",
)


### Unofficial datasets ###

XQUAD_DE_CONFIG = DatasetConfig(
    name="xquad-de",
    pretty_name="XQuAD-de",
    source="EuroEval/xquad-de",
    task=RC,
    languages=[GERMAN],
    unofficial=True,
)

ARC_DE_CONFIG = DatasetConfig(
    name="arc-de",
    pretty_name="ARC-de",
    source="EuroEval/arc-de-mini",
    task=KNOW,
    languages=[GERMAN],
    unofficial=True,
)

BELEBELE_DE_CONFIG = DatasetConfig(
    name="belebele-de",
    pretty_name="Belebele-de",
    source="EuroEval/belebele-de-mini",
    task=MCRC,
    languages=[GERMAN],
    unofficial=True,
)

MULTI_WIKI_QA_DE_CONFIG = DatasetConfig(
    name="multi-wiki-qa-de",
    pretty_name="MultiWikiQA-de",
    source="EuroEval/multi-wiki-qa-de-mini",
    task=RC,
    languages=[GERMAN],
    unofficial=True,
)

GOLDENSWAG_DE_CONFIG = DatasetConfig(
    name="goldenswag-de",
    pretty_name="GoldenSwag-de",
    source="EuroEval/goldenswag-de-mini",
    task=COMMON_SENSE,
    languages=[GERMAN],
    unofficial=True,
)

WINOGRANDE_DE_CONFIG = DatasetConfig(
    name="winogrande-de",
    pretty_name="Winogrande-de",
    source="EuroEval/winogrande-de",
    task=COMMON_SENSE,
    languages=[GERMAN],
    _labels=["a", "b"],
    unofficial=True,
)



"""All Serbian dataset configurations used in EuroEval."""

from ..data_models import DatasetConfig
from ..languages import SERBIAN
from ..tasks import COMMON_SENSE, KNOW, LA, NER, RC, SENT, SUMM

### Official datasets ###

MMS_SR_CONFIG = DatasetConfig(
    name="mms-sr",
    pretty_name="MMS-sr",
    source="EuroEval/mms-sr-mini",
    task=SENT,
    languages=[SERBIAN],
)

SCALA_SR_CONFIG = DatasetConfig(
    name="scala-sr",
    pretty_name="ScaLA-sr",
    source="EuroEval/scala-sr",
    task=LA,
    languages=[SERBIAN],
)

UNER_SR_CONFIG = DatasetConfig(
    name="uner-sr",
    pretty_name="UNER-sr",
    source="EuroEval/uner-sr-mini",
    task=NER,
    languages=[SERBIAN],
)

MULTI_WIKI_QA_SR_CONFIG = DatasetConfig(
    name="multi-wiki-qa-sr",
    pretty_name="MultiWikiQA-sr",
    source="EuroEval/multi-wiki-qa-sr-mini",
    task=RC,
    languages=[SERBIAN],
)

LR_SUM_SR_CONFIG = DatasetConfig(
    name="lr-sum-sr",
    pretty_name="LRSum-sr",
    source="EuroEval/lr-sum-sr-mini",
    task=SUMM,
    languages=[SERBIAN],
)

MMLU_SR_CONFIG = DatasetConfig(
    name="mmlu-sr",
    pretty_name="MMLU-sr",
    source="EuroEval/mmlu-sr-mini",
    task=KNOW,
    languages=[SERBIAN],
)

WINOGRANDE_SR_CONFIG = DatasetConfig(
    name="winogrande-sr",
    pretty_name="Winogrande-sr",
    source="EuroEval/winogrande-sr",
    task=COMMON_SENSE,
    languages=[SERBIAN],
    _labels=["a", "b"],
)



# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==3.5.0",
#     "huggingface-hub==0.24.0",
#     "pandas==2.2.0",
#     "requests==2.32.3",
# ]
# ///

"""Create the Finnish Turku NER dataset and upload it to the HF Hub."""

import pandas as pd
import requests
from datasets import Dataset, DatasetDict, Split
from huggingface_hub import HfApi

LABEL_MAPS = {
    "B-PRO": "B-MISC",
    "I-PRO": "I-MISC",
    "B-DATE": "B-MISC",
    "I-DATE": "I-MISC",
    "I-EVENT": "I-MISC",
    "B-EVENT": "B-MISC",
}


def main() -> None:
    """Create the Finnish Turku NER dataset and uploads it to the HF Hub."""
    # Read the CoNLL TSV files
    dfs = read_dataset()
    train_df = dfs["train"]
    val_df = dfs["val"]
    test_df = dfs["test"]

    # Add a `text` column
    train_df["text"] = train_df["tokens"].map(lambda tokens: " ".join(tokens))
    val_df["text"] = val_df["tokens"].map(lambda tokens: " ".join(tokens))
    test_df["text"] = test_df["tokens"].map(lambda tokens: " ".join(tokens))

    # Create validation split
    val_size = 256
    val_df = val_df.sample(n=min(val_size, len(val_df)), random_state=4242)

    # Create test split
    test_size = 2048
    test_df = test_df.sample(n=min(test_size, len(test_df)), random_state=4242)

    # Create train split
    train_size = 1024
    train_df = train_df.sample(n=min(train_size, len(train_df)), random_state=4242)

    # Reset the index
    train_df = train_df.reset_index(drop=True)
    val_df = val_df.reset_index(drop=True)
    test_df = test_df.reset_index(drop=True)

    # Collect datasets in a dataset dictionary
    dataset = DatasetDict(
        {
            "train": Dataset.from_pandas(train_df, split=Split.TRAIN),
            "val": Dataset.from_pandas(val_df, split=Split.VALIDATION),
            "test": Dataset.from_pandas(test_df, split=Split.TEST),
        }
    )

    # Push the dataset to the Hugging Face Hub
    dataset_id = "EuroEval/turku-ner-fi-mini"
    HfApi().delete_repo(dataset_id, repo_type="dataset", missing_ok=True)
    dataset.push_to_hub(dataset_id, private=True)


def read_dataset() -> dict:
    """Read the CoNLL-formatted TSV files and convert them to the required format."""
    base_url = (
        "https://raw.githubusercontent.com/TurkuNLP/turku-ner-corpus/refs/heads/"
        "master/data/conll/{}.tsv"
    )
    train_url = base_url.format("train")
    val_url = base_url.format("dev")
    test_url = base_url.format("test")

    data = dict(
        train=requests.get(train_url).text.split("\n\n"),
        val=requests.get(val_url).text.split("\n\n"),
        test=requests.get(test_url).text.split("\n\n"),
    )

    dfs = {}

    for split_name, split_raw_data in data.items():
        split_samples: list[dict[str, list[str]]] = []

        for raw_sample in split_raw_data:
            if "DOCSTART" in raw_sample or not raw_sample:
                continue

            tokens = []
            labels = []
            for line in raw_sample.split("\n"):
                token, label = line.split("\t")

                # Turku has additional labels that
                # we will not use.
                if label in LABEL_MAPS:
                    label = LABEL_MAPS[label]

                tokens.append(token)
                labels.append(label)

            split_samples.append({"tokens": tokens, "labels": labels})

        dfs[split_name] = pd.DataFrame.from_records(split_samples)

    # According to the paper, there should be 12,217 / 1,364 / 1,555
    # samples in the train/val/test splits, respectively.
    assert len(dfs["train"]) == 12_217
    assert len(dfs["val"]) == 1_364
    assert len(dfs["test"]) == 1_555

    return dfs


if __name__ == "__main__":
    main()



# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==3.5.0",
#     "huggingface-hub==0.24.0",
#     "pandas==2.2.0",
#     "requests==2.32.3",
# ]
# ///

"""Create the Winogrande datasets and upload them to the HF Hub."""

import logging
import re
from collections import Counter

import pandas as pd
from datasets import Dataset, DatasetDict, Split, disable_progress_bars, load_dataset
from huggingface_hub import HfApi

from .constants import (
    CHOICES_MAPPING,
    MAX_NUM_CHARS_IN_INSTRUCTION,
    MAX_NUM_CHARS_IN_OPTION,
    MAX_REPETITIONS,
    MIN_NUM_CHARS_IN_INSTRUCTION,
    MIN_NUM_CHARS_IN_OPTION,
)

logging.basicConfig(level=logging.INFO, format="%(asctime)s - %(message)s")
logger = logging.getLogger("create_winogrande")


LANGUAGES = [
    "bg",
    "ca",
    "da",
    "de",
    "el",
    "en",
    "es",
    "fi",
    "fr",
    "hr",
    "hu",
    "it",
    "lt",
    "lv",
    "nl",
    "no",
    "ro",
    "pl",
    "pt",
    "sk",
    "sl",
    "sr",
    "sv",
    "sq",
    "uk",
]


def main() -> None:
    """Create the Winogrande datasets and upload them to the HF Hub."""
    disable_progress_bars()
    repo_id = "aialt/MuBench"

    for language in LANGUAGES:
        # Download the dataset
        dataset = load_dataset(
            path=repo_id, name=f"WinoGrandeDataset_local_template_{language}"
        )
        assert isinstance(dataset, DatasetDict)

        # Convert the dataset to a dataframe
        train_df = dataset["train"].to_pandas()
        test_df = dataset["test"].to_pandas()
        assert isinstance(train_df, pd.DataFrame)
        assert isinstance(test_df, pd.DataFrame)

        # Split test set into validation and test sets
        val_size = 128
        val_df = test_df.sample(n=val_size, random_state=42).reset_index(drop=True)
        test_df = test_df.drop(val_df.index.tolist()).reset_index(drop=True)

        train_df = prepare_dataframe(df=train_df, language=language)
        val_df = prepare_dataframe(df=val_df, language=language)
        test_df = prepare_dataframe(df=test_df, language=language)

        # Collect datasets in a dataset dictionary
        dataset = DatasetDict(
            {
                "train": Dataset.from_pandas(train_df, split=Split.TRAIN),
                "val": Dataset.from_pandas(val_df, split=Split.VALIDATION),
                "test": Dataset.from_pandas(test_df, split=Split.TEST),
            }
        )

        logger.info(
            f"Final sizes for the Winogrande {language} dataset: "
            f"{len(dataset['train'])} train, {len(dataset['val'])} val, "
            f"{len(dataset['test'])} test"
        )

        # Push the dataset to the Hugging Face Hub
        target_dataset_id = f"EuroEval/winogrande-{language}"
        HfApi().delete_repo(target_dataset_id, repo_type="dataset", missing_ok=True)
        dataset.push_to_hub(target_dataset_id, private=True)


def prepare_dataframe(df: pd.DataFrame, language: str) -> pd.DataFrame:
    """Prepare the dataframe by filtering and processing.

    Args:
        df:
            Input dataframe.
        language:
            The language code that we are processing the dataframe for.

    Returns:
        Processed dataframe.
    """
    df.rename(columns=dict(prompt="instruction"), inplace=True)

    # Double check that there are exactly 6 lines in the instructions. These are:
    # 1. The instruction itself
    # 2. The equivalent of "What does the blank _ refer to?"
    # 3. The equivalent of "Option A: (...)"
    # 4. The equivalent of "Option B: (...)"
    # 5. "Answer with A or B."
    # 6. "Answer:"
    # We want to keep the actual instruction (lines 1-2) as well as the choices (lines
    # 3-4, without the "Option X" prefix), so we will trim the rest away.
    assert df.instruction.map(lambda x: x.count("\n")).eq(5).all(), (
        "Not all instructions have exactly 6 lines!"
    )
    df["option_a"] = df.instruction.map(
        lambda x: re.sub(r"[^ ]+ A: ?", "", x.split("\n")[2]).strip()
    )
    df["option_b"] = df.instruction.map(
        lambda x: re.sub(r"[^ ]+ B: ?", "", x.split("\n")[3]).strip()
    )
    df.instruction = df.instruction.map(lambda x: " ".join(x.split("\n")[:2]).strip())
    assert isinstance(df, pd.DataFrame)

    # Remove the samples with overly short or long texts
    df = df.loc[
        (df.instruction.str.len() >= MIN_NUM_CHARS_IN_INSTRUCTION)
        & (df.instruction.str.len() <= MAX_NUM_CHARS_IN_INSTRUCTION)
        & (df.option_a.str.len() >= MIN_NUM_CHARS_IN_OPTION)
        & (df.option_a.str.len() <= MAX_NUM_CHARS_IN_OPTION)
        & (df.option_b.str.len() >= MIN_NUM_CHARS_IN_OPTION)
        & (df.option_b.str.len() <= MAX_NUM_CHARS_IN_OPTION)
    ]

    # Remove overly repetitive samples
    df = df.loc[
        ~df.instruction.apply(is_repetitive)
        & ~df.option_a.apply(is_repetitive)
        & ~df.option_b.apply(is_repetitive)
    ]

    # Make a `text` column with all the options in it
    df["text"] = [
        row.instruction.replace("\n", " ").strip() + "\n"
        f"{CHOICES_MAPPING[language]}:\n"
        "a. " + row.option_a.replace("\n", " ").strip() + "\n"
        "b. " + row.option_b.replace("\n", " ").strip()
        for _, row in df.iterrows()
    ]

    # Make the `label` column case-consistent with the `text` column
    df.label = df.label.map(lambda x: "a" if x == 0 else "b")

    # Only keep the `text` and `label` columns
    df = df.loc[:, ["text", "label"]]

    # Remove duplicates
    df.drop_duplicates(inplace=True)
    df.reset_index(drop=True, inplace=True)

    return df


def is_repetitive(text: str) -> bool:
    """Return True if the text is repetitive."""
    max_repetitions = max(Counter(text.split()).values())
    return max_repetitions > MAX_REPETITIONS


if __name__ == "__main__":
    main()



# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==3.5.0",
#     "huggingface-hub==0.24.0",
#     "pandas==2.2.0",
#     "requests==2.32.3",
# ]
# ///

"""Create the Slovak CSFD sentiment dataset and upload to HF Hub."""

import pandas as pd
from datasets import Dataset, DatasetDict, Split, load_dataset
from huggingface_hub import HfApi


def main() -> None:
    """Create the Slovak CSFD sentiment dataset and upload to HF Hub."""
    # Define the dataset repository
    repo_id = "fewshot-goes-multilingual/sk_csfd-movie-reviews"

    # Download the dataset
    dataset = load_dataset(path=repo_id)
    assert isinstance(dataset, DatasetDict)

    # Process each split separately
    processed_splits = {}

    for split_name in ["train", "validation", "test"]:
        # Convert to dataframe
        df = dataset[split_name].to_pandas()
        assert isinstance(df, pd.DataFrame)

        # Keep only columns comment and rating_int for sentiment analysis
        df = df.loc[["comment", "rating_int"]]

        # Map rating_int to sentiment labels
        # ratings: 0-1 = negative, 2-3 = neutral, 4-5 = positive
        df["label"] = df["rating_int"].apply(
            lambda x: "negative" if x <= 1 else ("neutral" if x <= 3 else "positive")
        )

        # Rename comment column to text
        df.rename(columns={"comment": "text"}, inplace=True)

        # Keep only columns text and label
        df = df[["text", "label"]]

        # Remove duplicates within each split
        df = df.drop_duplicates().reset_index(drop=True)

        # Strip trailing whitespace
        df.text = df.text.str.strip()

        # Store processed split
        processed_splits[split_name] = df

    # Make splits
    train_df = processed_splits["train"]
    val_df = processed_splits["validation"]
    test_df = processed_splits["test"]

    # Define split sizes
    test_size = 2048
    val_size = 256
    train_size = 1024

    # Sample final splits
    final_test_df = test_df.sample(n=test_size, random_state=4242).reset_index(
        drop=True
    )
    final_val_df = val_df.sample(n=val_size, random_state=4242).reset_index(drop=True)
    final_train_df = train_df.sample(n=train_size, random_state=4242).reset_index(
        drop=True
    )

    # Create dataset dictionary with custom splits
    dataset = DatasetDict(
        {
            "train": Dataset.from_pandas(final_train_df, split=Split.TRAIN),
            "val": Dataset.from_pandas(final_val_df, split=Split.VALIDATION),
            "test": Dataset.from_pandas(final_test_df, split=Split.TEST),
        }
    )

    # Push the dataset to the Hugging Face Hub
    dataset_id = "EuroEval/csfd-sentiment-sk-mini"
    HfApi().delete_repo(dataset_id, repo_type="dataset", missing_ok=True)
    dataset.push_to_hub(dataset_id, private=True)


if __name__ == "__main__":
    main()



# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==3.5.0",
#     "huggingface-hub==0.24.0",
#     "pandas==2.2.0",
#     "requests==2.32.3",
# ]
# ///

"""Create the RoNEC NER dataset and upload it to the HF Hub."""

import pandas as pd
from datasets import Dataset, DatasetDict, load_dataset
from huggingface_hub import HfApi

NER_CONVERSION_DICT = {
    1: "B-PER",
    2: "I-PER",
    3: "B-ORG",
    4: "I-ORG",
    5: "B-LOC",
    6: "I-LOC",
    7: "B-MISC",
    8: "I-MISC",
}


def main() -> None:
    """Create the RoNEC NER dataset and upload it to the HF Hub."""
    # Define dataset ID
    repo_id = "community-datasets/ronec"

    # Download the dataset
    dataset = load_dataset(path=repo_id, token=True)
    assert isinstance(dataset, DatasetDict)

    # Convert the dataset to a dataframe
    train_df, val_df, test_df = convert_to_dataframe(dataset=dataset)

    # Process the dataframes
    train_df = process(df=train_df)
    val_df = process(df=val_df)
    test_df = process(df=test_df)

    # Make splits
    train_df_final, val_df_final, test_df_final = make_splits(
        train_df=train_df, val_df=val_df, test_df=test_df
    )

    # Create dataset dictionary
    mini_dataset = DatasetDict(
        {
            "train": Dataset.from_pandas(train_df_final),
            "val": Dataset.from_pandas(val_df_final),
            "test": Dataset.from_pandas(test_df_final),
        }
    )

    # Create dataset ID
    dataset_id = "EuroEval/ronec-mini"

    # Remove the dataset from Hugging Face Hub if it already exists
    HfApi().delete_repo(dataset_id, repo_type="dataset", missing_ok=True)

    # Push the dataset to the Hugging Face Hub
    mini_dataset.push_to_hub(dataset_id, private=True)


def convert_to_dataframe(
    dataset: DatasetDict,
) -> tuple[pd.DataFrame, pd.DataFrame, pd.DataFrame]:
    """Convert the dataset to a dataframe.

    Args:
        dataset: The dataset to convert.

    Returns:
        The training, validation, and test dataframes.
    """
    train_df = dataset["train"].to_pandas()
    val_df = dataset["validation"].to_pandas()
    test_df = dataset["test"].to_pandas()
    assert isinstance(train_df, pd.DataFrame)
    assert isinstance(val_df, pd.DataFrame)
    assert isinstance(test_df, pd.DataFrame)
    return train_df, val_df, test_df


def process(df: pd.DataFrame) -> pd.DataFrame:
    """Process the dataframe.

    Args:
        df:
            The dataframe to process.

    Returns:
        The processed dataframe.
    """
    # Sanity check that the number of tokens and named entity tags are equal
    assert all(
        len(ner_tags) == len(tokens)
        for ner_tags, tokens in zip(df["ner_tags"], df["tokens"])
    ), "The length of `ner_tags` and `tokens` are not equal in each row."

    # Rename `ner_tags` to `labels`
    df.rename(columns={"ner_tags": "labels"}, inplace=True)

    # Convert the NER tags from IDs to strings
    df["labels"] = df["labels"].map(
        lambda ner_tags: [NER_CONVERSION_DICT.get(ner_tag, "O") for ner_tag in ner_tags]
    )

    # Keep only tokens and labels columns
    keep_columns = ["tokens", "labels"]
    df = df.loc[keep_columns]

    return df


def make_splits(
    train_df: pd.DataFrame, val_df: pd.DataFrame, test_df: pd.DataFrame
) -> tuple[pd.DataFrame, pd.DataFrame, pd.DataFrame]:
    """Make splits for the dataset.

    Args:
        train_df: The training dataframe.
        val_df: The validation dataframe.
        test_df: The test dataframe.

    Returns:
        The final training, validation, and test dataframes.
    """
    # Sizes
    train_size = 1024
    val_size = 256
    test_size = 2048

    # Train
    train_df_final = train_df.sample(n=train_size, random_state=4242)

    # Val
    val_df_final = val_df.sample(n=val_size, random_state=4242)
    remaining_val_df = val_df[~val_df.index.isin(val_df_final.index)]

    # Test
    n_missing_test_samples = test_size - len(test_df)
    additional_test_samples = remaining_val_df.sample(
        n=n_missing_test_samples, random_state=4242
    )
    test_df_final = pd.concat([test_df, additional_test_samples], ignore_index=True)

    # Reset indices
    train_df_final = train_df_final.reset_index(drop=True)
    val_df_final = val_df_final.reset_index(drop=True)
    test_df_final = test_df_final.reset_index(drop=True)

    assert isinstance(train_df_final, pd.DataFrame)
    assert isinstance(val_df_final, pd.DataFrame)
    assert isinstance(test_df_final, pd.DataFrame)

    return train_df_final, val_df_final, test_df_final


if __name__ == "__main__":
    main()



"""General fixtures used throughout test modules."""

import collections.abc as c
import os
import sys
from pathlib import Path
from typing import Generator

import pytest
import torch
from click import ParamType

from euroeval.cli import benchmark
from euroeval.data_models import BenchmarkConfig, DatasetConfig, ModelConfig, Task
from euroeval.dataset_configs import get_all_dataset_configs
from euroeval.enums import InferenceBackend, ModelType
from euroeval.languages import DANISH, get_all_languages
from euroeval.metrics import HuggingFaceMetric
from euroeval.tasks import SENT


def pytest_configure() -> None:
    """Set a global flag when `pytest` is being run."""
    setattr(sys, "_called_from_test", True)
    if torch.cuda.is_available():
        os.environ["CUDA_VISIBLE_DEVICES"] = "0"  # Ensure only one GPU is used in tests


def pytest_unconfigure() -> None:
    """Unset the global flag when `pytest` is finished."""
    delattr(sys, "_called_from_test")


if os.environ.get("CHECK_DATASET") is not None:
    dataset_configs = [
        dataset_config
        for dataset_config in get_all_dataset_configs(
            custom_datasets_file=Path("custom_datasets.py")
        ).values()
        if dataset_config.name in os.environ["CHECK_DATASET"].split(",")
        or any(
            language.code in os.environ["CHECK_DATASET"].split(",")
            for language in dataset_config.languages
        )
        or "all" in os.environ["CHECK_DATASET"].split(",")
    ]
    ACTIVE_LANGUAGES = {
        language_code: language
        for language_code, language in get_all_languages().items()
        if any(language in cfg.languages for cfg in dataset_configs)
    }
else:
    ACTIVE_LANGUAGES = dict(da=DANISH)


@pytest.fixture(scope="session")
def auth() -> Generator[str | bool, None, None]:
    """Yields the authentication token to the Hugging Face Hub."""
    # Get the authentication token to the Hugging Face Hub
    auth = os.environ.get("HUGGINGFACE_API_KEY", True)

    # Ensure that the token does not contain quotes or whitespace
    if isinstance(auth, str):
        auth = auth.strip(" \"'")

    yield auth


@pytest.fixture(scope="session")
def device() -> Generator[torch.device, None, None]:
    """Yields the device to use for the tests."""
    if torch.cuda.is_available():
        device = torch.device("cuda")
    elif torch.backends.mps.is_available():
        device = torch.device("mps")
    else:
        device = torch.device("cpu")
    yield device


@pytest.fixture(scope="session")
def benchmark_config(
    auth: str, device: torch.device
) -> Generator[BenchmarkConfig, None, None]:
    """Yields a benchmark configuration used in tests."""
    yield BenchmarkConfig(
        languages=[DANISH],
        datasets=list(
            get_all_dataset_configs(
                custom_datasets_file=Path("custom_datasets.py")
            ).values()
        ),
        finetuning_batch_size=1,
        raise_errors=False,
        cache_dir=".euroeval_cache",
        api_key=auth,
        force=False,
        progress_bar=False,
        save_results=True,
        device=device,
        verbose=False,
        trust_remote_code=True,
        clear_model_cache=False,
        evaluate_test_split=False,
        few_shot=True,
        num_iterations=1,
        api_base=None,
        api_version=None,
        gpu_memory_utilization=0.8,
        attention_backend=None,
        generative_type=None,
        debug=False,
        run_with_cli=True,
        requires_safetensors=False,
        download_only=False,
    )


@pytest.fixture(scope="session")
def metric() -> Generator[HuggingFaceMetric, None, None]:
    """Yields a metric configuration used in tests."""
    yield HuggingFaceMetric(
        name="metric_name",
        pretty_name="Metric name",
        huggingface_id="metric_id",
        results_key="metric_key",
    )


@pytest.fixture(
    scope="session",
    params=list(
        {
            dataset_config.task
            for dataset_config in get_all_dataset_configs(
                custom_datasets_file=Path("custom_datasets.py")
            ).values()
        }
    ),
    ids=lambda task: task.name,
)
def task(request: pytest.FixtureRequest) -> Generator[Task, None, None]:
    """Yields a dataset task used in tests."""
    yield request.param


@pytest.fixture(
    scope="session",
    params=list(ACTIVE_LANGUAGES.values()),
    ids=list(ACTIVE_LANGUAGES.keys()),
)
def language(request: pytest.FixtureRequest) -> Generator[str, None, None]:
    """Yields a language used in tests."""
    yield request.param


@pytest.fixture(scope="session")
def encoder_model_id() -> Generator[str, None, None]:
    """Yields a model ID used in tests."""
    yield "jonfd/electra-small-nordic"


@pytest.fixture(scope="session")
def generative_model_id() -> Generator[str, None, None]:
    """Yields a generative model ID used in tests."""
    yield "HuggingFaceTB/SmolLM2-135M"


@pytest.fixture(scope="session")
def generative_adapter_model_id() -> Generator[str, None, None]:
    """Yields a generative adapter model ID used in tests."""
    yield "jekunz/smollm-135m-lora-fineweb-swedish"


@pytest.fixture(scope="session")
def openai_model_id() -> Generator[str, None, None]:
    """Yields an OpenAI model ID used in tests."""
    yield "gpt-4o-mini"


@pytest.fixture(scope="session")
def ollama_model_id() -> Generator[str, None, None]:
    """Yields an Ollama model ID used in tests."""
    yield "ollama_chat/smollm2:135m"


@pytest.fixture(scope="session")
def model_config() -> Generator[ModelConfig, None, None]:
    """Yields a model configuration used in tests."""
    yield ModelConfig(
        model_id="model_id",
        revision="revision",
        param=None,
        task="task",
        languages=[DANISH],
        merge=False,
        inference_backend=InferenceBackend.TRANSFORMERS,
        model_type=ModelType.ENCODER,
        fresh=True,
        model_cache_dir="cache_dir",
        adapter_base_model_id=None,
    )


@pytest.fixture(scope="module")
def cli_params() -> Generator[dict[str | None, ParamType], None, None]:
    """Yields a dictionary of the CLI parameters."""
    ctx = benchmark.make_context(info_name="testing", args=["--model", "test-model"])
    yield {p.name: p.type for p in benchmark.get_params(ctx)}


@pytest.fixture(scope="session")
def dataset_config() -> c.Generator[DatasetConfig, None, None]:
    """Yields a dataset configuration used in tests."""
    yield DatasetConfig(
        name="dataset",
        pretty_name="Dataset",
        source="dataset_id",
        task=SENT,
        languages=[DANISH],
    )



"""All Dutch dataset configurations used in EuroEval."""

from ..data_models import DatasetConfig
from ..languages import DUTCH
from ..tasks import (
    COMMON_SENSE,
    EUROPEAN_VALUES,
    KNOW,
    LA,
    MCRC,
    MCSTEREO,
    NER,
    RC,
    SENT,
    SIMPL,
    SUMM,
)

### Official datasets ###

DBRD_CONFIG = DatasetConfig(
    name="dbrd",
    pretty_name="DBRD",
    source="EuroEval/dbrd-mini",
    task=SENT,
    languages=[DUTCH],
    _labels=["negative", "positive"],
)

SCALA_NL_CONFIG = DatasetConfig(
    name="scala-nl",
    pretty_name="ScaLA-nl",
    source="EuroEval/scala-nl",
    task=LA,
    languages=[DUTCH],
)

CONLL_NL_CONFIG = DatasetConfig(
    name="conll-nl",
    pretty_name="CoNLL-nl",
    source="EuroEval/conll-nl-mini",
    task=NER,
    languages=[DUTCH],
)

SQUAD_NL_CONFIG = DatasetConfig(
    name="squad-nl",
    pretty_name="SQuAD-nl",
    source="EuroEval/squad-nl-v2-mini",
    task=RC,
    languages=[DUTCH],
)

WIKI_LINGUA_NL_CONFIG = DatasetConfig(
    name="wiki-lingua-nl",
    pretty_name="WikiLingua-nl",
    source="EuroEval/wiki-lingua-nl-mini",
    task=SUMM,
    languages=[DUTCH],
)

MMLU_NL_CONFIG = DatasetConfig(
    name="mmlu-nl",
    pretty_name="MMLU-nl",
    source="EuroEval/mmlu-nl-mini",
    task=KNOW,
    languages=[DUTCH],
)

HELLASWAG_NL_CONFIG = DatasetConfig(
    name="hellaswag-nl",
    pretty_name="HellaSwag-nl",
    source="EuroEval/hellaswag-nl-mini",
    task=COMMON_SENSE,
    languages=[DUTCH],
)

DUIDELIJKE_TAAL_NL_CONFIG = DatasetConfig(
    name="duidelijke-taal",
    pretty_name="Duidelijke Taal",
    source="EuroEval/duidelijke-taal",
    task=SIMPL,
    languages=[DUTCH],
)

VALEU_NL_CONFIG = DatasetConfig(
    name="valeu-nl",
    pretty_name="VaLEU-nl",
    source="EuroEval/european-values-nl",
    task=EUROPEAN_VALUES,
    languages=[DUTCH],
    splits=["test"],
    bootstrap_samples=False,
    _instruction_prompt="{text}",
)

MBBQ_NL_CONFIG = DatasetConfig(
    name="mbbq-nl",
    pretty_name="MBBQ-nl",
    source="EuroEval/mbbq-nl",
    task=MCSTEREO,
    languages=[DUTCH],
    splits=["val", "test"],
)


### Unofficial datasets ###

DUTCH_COLA_CONFIG = DatasetConfig(
    name="dutch-cola",
    pretty_name="Dutch CoLA",
    source="EuroEval/dutch-cola",
    task=LA,
    languages=[DUTCH],
    unofficial=True,
)

DUTCH_COLA_FULL_CONFIG = DatasetConfig(
    name="dutch-cola-full",
    pretty_name="Dutch CoLA Full",
    source="EuroEval/dutch-cola-full",
    task=LA,
    languages=[DUTCH],
    unofficial=True,
)

ARC_NL_CONFIG = DatasetConfig(
    name="arc-nl",
    pretty_name="ARC-nl",
    source="EuroEval/arc-nl-mini",
    task=KNOW,
    languages=[DUTCH],
    unofficial=True,
)

BELEBELE_NL_CONFIG = DatasetConfig(
    name="belebele-nl",
    pretty_name="Belebele-nl",
    source="EuroEval/belebele-nl-mini",
    task=MCRC,
    languages=[DUTCH],
    unofficial=True,
)

MULTI_WIKI_QA_NL_CONFIG = DatasetConfig(
    name="multi-wiki-qa-nl",
    pretty_name="MultiWikiQA-nl",
    source="EuroEval/multi-wiki-qa-nl-mini",
    task=RC,
    languages=[DUTCH],
    unofficial=True,
)

COPA_NL_CONFIG = DatasetConfig(
    name="copa-nl",
    pretty_name="COPA-nl",
    source="EuroEval/copa-nl",
    task=COMMON_SENSE,
    languages=[DUTCH],
    unofficial=True,
    _labels=["a", "b"],
)

GOLDENSWAG_NL_CONFIG = DatasetConfig(
    name="goldenswag-nl",
    pretty_name="GoldenSwag-nl",
    source="EuroEval/goldenswag-nl-mini",
    task=COMMON_SENSE,
    languages=[DUTCH],
    unofficial=True,
)

WINOGRANDE_NL_CONFIG = DatasetConfig(
    name="winogrande-nl",
    pretty_name="Winogrande-nl",
    source="EuroEval/winogrande-nl",
    task=COMMON_SENSE,
    languages=[DUTCH],
    _labels=["a", "b"],
    unofficial=True,
)



"""The different prompt templates used in EuroEval."""

from .classification import CLASSIFICATION_TEMPLATES
from .linguistic_acceptability import LA_TEMPLATES
from .multiple_choice import MULTIPLE_CHOICE_TEMPLATES
from .named_entity_recognition import NER_TEMPLATES
from .reading_comprehension import RC_TEMPLATES
from .sentiment_classification import SENT_TEMPLATES
from .simplification import SIMPL_TEMPLATES
from .summarization import SUMM_TEMPLATES
from .token_classification import TOKEN_CLASSIFICATION_TEMPLATES



# /// script
# requires-python = ">=3.10,<4.0"
# dependencies = [
#     "datasets==4.0.0",
#     "huggingface-hub==0.34.4",
#     "requests==2.32.5",
# ]
# ///
"""Create the Estonian Trivia dataset and upload to HF Hub."""

from typing import MutableMapping

from datasets import Dataset, DatasetDict, concatenate_datasets, load_dataset
from huggingface_hub import HfApi

from .constants import CHOICES_MAPPING


def main() -> None:
    """Create the Estonian Trivia dataset and upload to HF Hub."""
    target_repo_id = "EuroEval/trivia-et"

    # start from the official source
    ds = load_dataset("TalTechNLP/trivia_et", split="train")

    ds = ds.map(add_options_and_label)

    # examples that are OK to show in the documentation with answers
    doc_examples_ids = [96, 97, 98]

    doc_examples = ds.filter(lambda row: row["id"] in doc_examples_ids)
    ds = ds.filter(lambda row: row["id"] not in doc_examples_ids)
    ds = ds.shuffle(seed=42)

    assert isinstance(doc_examples, Dataset)
    assert isinstance(ds, Dataset)

    train_size = 240 - len(doc_examples)
    val_size = 60
    test_size = 500

    train_ds = ds.select(range(train_size))
    assert isinstance(train_ds, Dataset)
    train_ds = concatenate_datasets([train_ds, doc_examples])
    val_ds = ds.skip(train_size).select(range(val_size))
    test_ds = ds.skip(train_size + val_size).select(range(test_size))

    assert isinstance(train_ds, Dataset)
    assert isinstance(val_ds, Dataset)
    assert isinstance(test_ds, Dataset)

    ds = DatasetDict({"train": train_ds, "val": val_ds, "test": test_ds})

    ds = ds.select_columns(["text", "label"])

    # Push the dataset to the Hugging Face Hub
    HfApi().delete_repo(target_repo_id, repo_type="dataset", missing_ok=True)
    ds.push_to_hub(target_repo_id, private=True)


def add_options_and_label(row: MutableMapping) -> MutableMapping:
    """Add options to the text and transform labels to letters."""
    letters = ["a", "b", "c", "d"]

    question = row["küsimus"]
    answer = row["vastus"]
    options = row["vastusevariandid"]

    options = [f"{letters[i]}. {options[i]}" for i in range(len(options))]
    options = "\n".join(options)

    text = f"{question}\n{CHOICES_MAPPING['et']}:\n{options}"
    label = letters[answer]

    return {"text": text, "label": label}


if __name__ == "__main__":
    main()
